'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var PropTypes = require('prop-types');
var cs = require('classnames');
var _$1 = require('underscore');
var reactTransitionGroup = require('react-transition-group');
var EventEmitter = require('events');

function _interopDefault (e) { return e && e.__esModule ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefault(React);
var PropTypes__default = /*#__PURE__*/_interopDefault(PropTypes);
var cs__default = /*#__PURE__*/_interopDefault(cs);
var ___default = /*#__PURE__*/_interopDefault(_$1);
var EventEmitter__default = /*#__PURE__*/_interopDefault(EventEmitter);

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

const ROW_SELECT_DISABLED = 'ROW_SELECT_DISABLED';
var Const = {
  SORT_ASC: 'asc',
  SORT_DESC: 'desc',
  ROW_SELECT_SINGLE: 'radio',
  ROW_SELECT_MULTIPLE: 'checkbox',
  ROW_SELECT_DISABLED,
  CHECKBOX_STATUS_CHECKED: 'checked',
  CHECKBOX_STATUS_INDETERMINATE: 'indeterminate',
  CHECKBOX_STATUS_UNCHECKED: 'unchecked',
  INDICATOR_POSITION_LEFT: 'left',
  INDICATOR_POSITION_RIGHT: 'right',
  TYPE_STRING: 'string',
  TYPE_NUMBER: 'number',
  TYPE_BOOLEAN: 'bool',
  TYPE_DATE: 'date',
  FILTERS_POSITION_INLINE: 'inline',
  FILTERS_POSITION_TOP: 'top',
  FILTERS_POSITION_BOTTOM: 'bottom'
};

const BootstrapContext = /*#__PURE__*/React__default['default'].createContext({
  bootstrap4: false
});

const SortSymbol = () => /*#__PURE__*/React__default['default'].createElement(BootstrapContext.Consumer, null, ({
  bootstrap4
}) => bootstrap4 ? /*#__PURE__*/React__default['default'].createElement("span", {
  className: "order-4"
}) : /*#__PURE__*/React__default['default'].createElement("span", {
  className: "order"
}, /*#__PURE__*/React__default['default'].createElement("span", {
  className: "dropdown"
}, /*#__PURE__*/React__default['default'].createElement("span", {
  className: "caret"
})), /*#__PURE__*/React__default['default'].createElement("span", {
  className: "dropup"
}, /*#__PURE__*/React__default['default'].createElement("span", {
  className: "caret"
}))));

const SortCaret = ({
  order
}) => {
  const orderClass = cs__default['default']('react-bootstrap-table-sort-order', {
    dropup: order === Const.SORT_ASC
  });
  return /*#__PURE__*/React__default['default'].createElement(BootstrapContext.Consumer, null, ({
    bootstrap4
  }) => bootstrap4 ? /*#__PURE__*/React__default['default'].createElement("span", {
    className: `caret-4-${order}`
  }) : /*#__PURE__*/React__default['default'].createElement("span", {
    className: orderClass
  }, /*#__PURE__*/React__default['default'].createElement("span", {
    className: "caret"
  })));
};

SortCaret.propTypes = {
  order: PropTypes__default['default'].oneOf([Const.SORT_ASC, Const.SORT_DESC]).isRequired
};

/* eslint no-empty: 0 */

function splitNested(str) {
  return [str].join('.').replace(/\[/g, '.').replace(/\]/g, '').split('.');
}

function contains(list, value) {
  if (___default['default'].includes) {
    return ___default['default'].includes(list, value);
  }

  return list.indexOf(value) > -1;
}

function get(target, field) {
  const directGet = target[field];

  if (directGet !== undefined && directGet !== null) {
    return directGet;
  }

  const pathArray = splitNested(field);
  let result;

  try {
    result = pathArray.reduce((curr, path) => curr[path], target);
  } catch (e) {}

  return result;
}

function set(target, field, value, safe = false) {
  const pathArray = splitNested(field);
  let level = 0;
  pathArray.reduce((a, b) => {
    level += 1;

    if (typeof a[b] === 'undefined') {
      if (!safe) throw new Error(`${a}.${b} is undefined`);
      a[b] = {};
      return a[b];
    }

    if (level === pathArray.length) {
      a[b] = value;
      return value;
    }

    return a[b];
  }, target);
}

function isEmptyObject(obj) {
  if (!___default['default'].isObject(obj)) return false;
  const {
    hasOwnProperty
  } = Object.prototype;
  const keys = Object.keys(obj);

  for (let i = 0; i < keys.length; i += 1) {
    if (hasOwnProperty.call(obj, keys[i])) return false;
  }

  return true;
}

function isDefined(value) {
  return typeof value !== 'undefined' && value !== null;
}

function sleep(fn, ms) {
  return setTimeout(() => fn(), ms);
}

function debounce(func, wait, immediate) {
  let timeout;
  return () => {
    const later = () => {
      timeout = null;

      if (!immediate) {
        func.apply(this, arguments);
      }
    };

    const callNow = immediate && !timeout;
    clearTimeout(timeout);
    timeout = setTimeout(later, wait || 0);

    if (callNow) {
      func.apply(this, arguments);
    }
  };
}

var _ = Object.assign(___default['default'], {
  get,
  set,
  isDefined,
  isEmptyObject,
  sleep,
  debounce,
  contains
});

const events = ['onClick', 'onDoubleClick', 'onMouseEnter', 'onMouseLeave', 'onContextMenu', 'onAuxClick'];
var eventDelegater = (ExtendBase => {
  var _temp;

  return _temp = class CellEventDelegater extends ExtendBase {
    constructor(...args) {
      super(...args);

      this.createDefaultEventHandler = cb => e => {
        const {
          column,
          columnIndex,
          index
        } = this.props;
        cb(e, column, typeof columnIndex !== 'undefined' ? columnIndex : index);
      };
    }

    delegate(attrs = {}) {
      const newAttrs = _objectSpread2({}, attrs);

      Object.keys(attrs).forEach(attr => {
        if (_.contains(events, attr)) {
          newAttrs[attr] = this.createDefaultEventHandler(attrs[attr]);
        }
      });
      return newAttrs;
    }

  }, _temp;
});

class HeaderCell extends eventDelegater(React__default['default'].Component) {
  render() {
    const {
      column,
      index,
      onSort,
      sorting,
      sortOrder,
      isLastSorting,
      onFilter,
      currFilters,
      filterPosition,
      onExternalFilter,
      globalSortCaret
    } = this.props;
    const {
      text,
      sort,
      sortCaret,
      filter,
      filterRenderer,
      headerTitle,
      headerAlign,
      headerFormatter,
      headerEvents,
      headerClasses,
      headerStyle,
      headerAttrs,
      headerSortingClasses,
      headerSortingStyle
    } = column;
    const sortCaretfunc = sortCaret || globalSortCaret;
    const delegateEvents = this.delegate(headerEvents);
    const customAttrs = _.isFunction(headerAttrs) ? headerAttrs(column, index) : headerAttrs || {};

    const cellAttrs = _objectSpread2(_objectSpread2(_objectSpread2({}, customAttrs), delegateEvents), {}, {
      tabIndex: _.isDefined(customAttrs.tabIndex) ? customAttrs.tabIndex : 0
    });

    let sortSymbol;
    let filterElm;
    let cellStyle = {};
    let cellClasses = _.isFunction(headerClasses) ? headerClasses(column, index) : headerClasses;

    if (headerStyle) {
      cellStyle = _.isFunction(headerStyle) ? headerStyle(column, index) : headerStyle;
      cellStyle = cellStyle ? _objectSpread2({}, cellStyle) : cellStyle;
    }

    if (headerTitle) {
      cellAttrs.title = _.isFunction(headerTitle) ? headerTitle(column, index) : text;
    }

    if (headerAlign) {
      cellStyle.textAlign = _.isFunction(headerAlign) ? headerAlign(column, index) : headerAlign;
    }

    if (sort) {
      const customClick = cellAttrs.onClick;
      cellAttrs['aria-label'] = sorting ? `${text} sort ${sortOrder}` : `${text} sortable`;

      cellAttrs.onKeyUp = e => {
        if (e.key === 'Enter') {
          onSort(column);
          if (_.isFunction(customClick)) customClick(e);
        }
      };

      cellAttrs.onClick = e => {
        onSort(column);
        if (_.isFunction(customClick)) customClick(e);
      };

      cellAttrs.className = cs__default['default'](cellAttrs.className, 'sortable');

      if (sorting) {
        sortSymbol = sortCaretfunc ? sortCaretfunc(sortOrder, column) : /*#__PURE__*/React__default['default'].createElement(SortCaret, {
          order: sortOrder
        }); // append customized classes or style if table was sorting based on the current column.

        cellClasses = cs__default['default'](cellClasses, _.isFunction(headerSortingClasses) ? headerSortingClasses(column, sortOrder, isLastSorting, index) : headerSortingClasses);
        cellStyle = _objectSpread2(_objectSpread2({}, cellStyle), _.isFunction(headerSortingStyle) ? headerSortingStyle(column, sortOrder, isLastSorting, index) : headerSortingStyle);
      } else {
        sortSymbol = sortCaretfunc ? sortCaretfunc(undefined, column) : /*#__PURE__*/React__default['default'].createElement(SortSymbol, null);
      }
    }

    if (cellClasses) cellAttrs.className = cs__default['default'](cellAttrs.className, cellClasses);
    if (!_.isEmptyObject(cellStyle)) cellAttrs.style = cellStyle;

    if (filterPosition === Const.FILTERS_POSITION_INLINE) {
      if (filterRenderer) {
        const onCustomFilter = onExternalFilter(column, filter.props.type);
        filterElm = filterRenderer(onCustomFilter, column);
      } else if (filter) {
        filterElm = /*#__PURE__*/React__default['default'].createElement(filter.Filter, _extends({}, filter.props, {
          filterState: currFilters[column.dataField],
          onFilter: onFilter,
          column: column
        }));
      }
    }

    const children = headerFormatter ? headerFormatter(column, index, {
      sortElement: sortSymbol,
      filterElement: filterElm
    }) : text;

    if (headerFormatter) {
      return /*#__PURE__*/React__default['default'].createElement('th', cellAttrs, children);
    }

    return /*#__PURE__*/React__default['default'].createElement('th', cellAttrs, children, sortSymbol, filterElm);
  }

}

HeaderCell.propTypes = {
  column: PropTypes__default['default'].shape({
    dataField: PropTypes__default['default'].string.isRequired,
    text: PropTypes__default['default'].string.isRequired,
    type: PropTypes__default['default'].oneOf([Const.TYPE_STRING, Const.TYPE_NUMBER, Const.TYPE_BOOLEAN, Const.TYPE_DATE]),
    isDummyField: PropTypes__default['default'].bool,
    hidden: PropTypes__default['default'].bool,
    headerFormatter: PropTypes__default['default'].func,
    formatter: PropTypes__default['default'].func,
    formatExtraData: PropTypes__default['default'].any,
    headerClasses: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].func]),
    classes: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].func]),
    headerStyle: PropTypes__default['default'].oneOfType([PropTypes__default['default'].object, PropTypes__default['default'].func]),
    style: PropTypes__default['default'].oneOfType([PropTypes__default['default'].object, PropTypes__default['default'].func]),
    headerTitle: PropTypes__default['default'].oneOfType([PropTypes__default['default'].bool, PropTypes__default['default'].func]),
    title: PropTypes__default['default'].oneOfType([PropTypes__default['default'].bool, PropTypes__default['default'].func]),
    headerEvents: PropTypes__default['default'].object,
    events: PropTypes__default['default'].object,
    headerAlign: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].func]),
    align: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].func]),
    headerAttrs: PropTypes__default['default'].oneOfType([PropTypes__default['default'].object, PropTypes__default['default'].func]),
    attrs: PropTypes__default['default'].oneOfType([PropTypes__default['default'].object, PropTypes__default['default'].func]),
    sort: PropTypes__default['default'].bool,
    sortFunc: PropTypes__default['default'].func,
    onSort: PropTypes__default['default'].func,
    editor: PropTypes__default['default'].object,
    editable: PropTypes__default['default'].oneOfType([PropTypes__default['default'].bool, PropTypes__default['default'].func]),
    editCellStyle: PropTypes__default['default'].oneOfType([PropTypes__default['default'].object, PropTypes__default['default'].func]),
    editCellClasses: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].func]),
    editorStyle: PropTypes__default['default'].oneOfType([PropTypes__default['default'].object, PropTypes__default['default'].func]),
    editorClasses: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].func]),
    editorRenderer: PropTypes__default['default'].func,
    validator: PropTypes__default['default'].func,
    filter: PropTypes__default['default'].object,
    filterRenderer: PropTypes__default['default'].func,
    filterValue: PropTypes__default['default'].func,
    searchable: PropTypes__default['default'].bool
  }).isRequired,
  index: PropTypes__default['default'].number.isRequired,
  onSort: PropTypes__default['default'].func,
  sorting: PropTypes__default['default'].bool,
  sortOrder: PropTypes__default['default'].oneOf([Const.SORT_ASC, Const.SORT_DESC]),
  sortCaret: PropTypes__default['default'].func,
  isLastSorting: PropTypes__default['default'].bool,
  onFilter: PropTypes__default['default'].func,
  filterPosition: PropTypes__default['default'].oneOf([Const.FILTERS_POSITION_INLINE, Const.FILTERS_POSITION_BOTTOM, Const.FILTERS_POSITION_TOP]),
  currFilters: PropTypes__default['default'].object,
  onExternalFilter: PropTypes__default['default'].func
};

const CheckBox = ({
  className,
  checked,
  indeterminate
}) => /*#__PURE__*/React__default['default'].createElement("input", {
  type: "checkbox",
  checked: checked,
  className: className,
  ref: input => {
    if (input) input.indeterminate = indeterminate; // eslint-disable-line no-param-reassign
  },
  onChange: () => {}
});
CheckBox.propTypes = {
  checked: PropTypes__default['default'].bool.isRequired,
  indeterminate: PropTypes__default['default'].bool.isRequired,
  className: PropTypes__default['default'].string
};

class SelectionHeaderCell extends React.Component {
  constructor(...args) {
    super(...args);

    this.handleCheckBoxClick = e => {
      const {
        onAllRowsSelect,
        checkedStatus
      } = this.props;
      const isUnSelect = checkedStatus === Const.CHECKBOX_STATUS_CHECKED || checkedStatus === Const.CHECKBOX_STATUS_INDETERMINATE;
      onAllRowsSelect(e, isUnSelect);
    };
  }

  /**
   * avoid updating if button is
   * 1. radio
   * 2. status was not changed.
   */
  shouldComponentUpdate(nextProps) {
    const {
      ROW_SELECT_SINGLE
    } = Const;
    const {
      mode,
      checkedStatus
    } = this.props;
    if (mode === ROW_SELECT_SINGLE) return false;
    return nextProps.checkedStatus !== checkedStatus;
  }

  render() {
    const {
      CHECKBOX_STATUS_CHECKED,
      CHECKBOX_STATUS_INDETERMINATE,
      ROW_SELECT_MULTIPLE
    } = Const;
    const {
      mode,
      checkedStatus,
      selectionHeaderRenderer,
      hideSelectAll,
      headerColumnStyle
    } = this.props;

    if (hideSelectAll) {
      // eslint-disable-next-line jsx-a11y/control-has-associated-label
      return /*#__PURE__*/React__default['default'].createElement("th", {
        "data-row-selection": true
      });
    }

    const checked = checkedStatus === CHECKBOX_STATUS_CHECKED;
    const indeterminate = checkedStatus === CHECKBOX_STATUS_INDETERMINATE;
    const attrs = {};
    let content;

    if (selectionHeaderRenderer || mode === ROW_SELECT_MULTIPLE) {
      attrs.onClick = this.handleCheckBoxClick;
    }

    attrs.style = _.isFunction(headerColumnStyle) ? headerColumnStyle(checkedStatus) : headerColumnStyle;
    return /*#__PURE__*/React__default['default'].createElement(BootstrapContext.Consumer, null, ({
      bootstrap4
    }) => {
      if (selectionHeaderRenderer) {
        content = selectionHeaderRenderer({
          mode,
          checked,
          indeterminate
        });
      } else if (mode === ROW_SELECT_MULTIPLE) {
        content = /*#__PURE__*/React__default['default'].createElement(CheckBox, _extends({}, this.props, {
          checked: checked,
          className: bootstrap4 ? 'selection-input-4' : '',
          indeterminate: indeterminate
        }));
      }

      return /*#__PURE__*/React__default['default'].createElement("th", _extends({
        className: "selection-cell-header",
        "data-row-selection": true
      }, attrs), content);
    });
  }

}

SelectionHeaderCell.propTypes = {
  mode: PropTypes__default['default'].string.isRequired,
  checkedStatus: PropTypes__default['default'].string,
  onAllRowsSelect: PropTypes__default['default'].func,
  hideSelectAll: PropTypes__default['default'].bool,
  selectionHeaderRenderer: PropTypes__default['default'].func,
  headerColumnStyle: PropTypes__default['default'].oneOfType([PropTypes__default['default'].object, PropTypes__default['default'].func])
};

class ExpansionHeaderCell extends React.Component {
  constructor(...args) {
    super(...args);

    this.handleCheckBoxClick = e => {
      const {
        isAnyExpands,
        onAllRowExpand
      } = this.props;
      onAllRowExpand(e, !isAnyExpands);
    };
  }

  render() {
    const {
      isAnyExpands,
      expandHeaderColumnRenderer
    } = this.props;
    const attrs = {
      onClick: this.handleCheckBoxClick
    };
    return (
      /*#__PURE__*/
      // eslint-disable-next-line react/jsx-props-no-spreading
      React__default['default'].createElement("th", _extends({
        className: "expand-cell-header",
        "data-row-selection": true
      }, attrs), expandHeaderColumnRenderer ? expandHeaderColumnRenderer({
        isAnyExpands
      }) : isAnyExpands ? '(-)' : '(+)')
    );
  }

}

ExpansionHeaderCell.propTypes = {
  isAnyExpands: PropTypes__default['default'].bool.isRequired,
  onAllRowExpand: PropTypes__default['default'].func.isRequired,
  expandHeaderColumnRenderer: PropTypes__default['default'].func
};

const matchRow = (keyField, id) => row => _.get(row, keyField) === id;
const getRowByRowId = (data, keyField, id) => data.find(matchRow(keyField, id));

var rows = /*#__PURE__*/Object.freeze({
  __proto__: null,
  matchRow: matchRow,
  getRowByRowId: getRowByRowId
});

const getSelectionSummary = (data = [], keyField, selected = []) => {
  let allRowsSelected = data.length > 0;
  let allRowsNotSelected = true;
  const rowKeys = data.map(d => _.get(d, keyField));

  for (let i = 0; i < rowKeys.length; i += 1) {
    const curr = rowKeys[i];

    if (typeof selected.find(x => x === curr) === 'undefined') {
      allRowsSelected = false;
    } else {
      allRowsNotSelected = false;
    }
  }

  return {
    allRowsSelected,
    allRowsNotSelected
  };
};
const selectableKeys = (data = [], keyField, skips = []) => {
  if (skips.length === 0) {
    return data.map(row => _.get(row, keyField));
  }

  return data.filter(row => !_.contains(skips, _.get(row, keyField))).map(row => _.get(row, keyField));
};
const unSelectableKeys = (selected, skips = []) => {
  if (skips.length === 0) {
    return [];
  }

  return selected.filter(x => _.contains(skips, x));
}; // eslint-disable-next-line max-len

const getSelectedRows = (data = [], keyField, selected) => selected.map(k => getRowByRowId(data, keyField, k)).filter(x => !!x);

var selection = /*#__PURE__*/Object.freeze({
  __proto__: null,
  getSelectionSummary: getSelectionSummary,
  selectableKeys: selectableKeys,
  unSelectableKeys: unSelectableKeys,
  getSelectedRows: getSelectedRows
});

const isAnyExpands = (data, keyField, expanded = []) => {
  for (let i = 0; i < data.length; i += 1) {
    const rowKey = _.get(data[i], keyField);

    if (typeof expanded.find(x => x === rowKey) !== 'undefined') {
      return true;
    }
  }

  return false;
};
const expandableKeys = (data, keyField, skips = []) => {
  if (skips.length === 0) {
    return data.map(row => _.get(row, keyField));
  }

  return data.filter(row => !_.contains(skips, _.get(row, keyField))).map(row => _.get(row, keyField));
}; // eslint-disable-next-line max-len

const getExpandedRows = (data, keyField, expanded) => expanded.map(k => getRowByRowId(data, keyField, k));

var expand = /*#__PURE__*/Object.freeze({
  __proto__: null,
  isAnyExpands: isAnyExpands,
  expandableKeys: expandableKeys,
  getExpandedRows: getExpandedRows
});

const editCell = (data, keyField, rowId, dataField, newValue) => {
  const row = getRowByRowId(data, keyField, rowId);
  if (row) _.set(row, dataField, newValue);
};

var mutate = /*#__PURE__*/Object.freeze({
  __proto__: null,
  editCell: editCell
});

/* eslint no-nested-ternary: 0 */

function comparator(a, b) {
  let result;

  if (typeof b === 'string') {
    result = b.localeCompare(a);
  } else {
    result = a > b ? -1 : a < b ? 1 : 0;
  }

  return result;
}

const sort = (data, sortOrder, {
  dataField,
  sortFunc,
  sortValue
}) => {
  const _data = [...data];

  _data.sort((a, b) => {
    let result;

    let valueA = _.get(a, dataField);

    let valueB = _.get(b, dataField);

    if (sortValue) {
      valueA = sortValue(valueA, a);
      valueB = sortValue(valueB, b);
    } else {
      valueA = _.isDefined(valueA) ? valueA : '';
      valueB = _.isDefined(valueB) ? valueB : '';
    }

    if (sortFunc) {
      result = sortFunc(valueA, valueB, sortOrder, dataField, a, b);
    } else {
      if (sortOrder === Const.SORT_DESC) {
        result = comparator(valueA, valueB);
      } else {
        result = comparator(valueB, valueA);
      }
    }

    return result;
  });

  return _data;
};
const nextOrder = (currentSortColumn, {
  sortOrder,
  sortColumn
}, defaultOrder = Const.SORT_DESC) => {
  if (!sortColumn || currentSortColumn.dataField !== sortColumn.dataField) return defaultOrder;
  return sortOrder === Const.SORT_DESC ? Const.SORT_ASC : Const.SORT_DESC;
};

var sort$1 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  sort: sort,
  nextOrder: nextOrder
});

const typeConvert = (type, value) => {
  if (type === Const.TYPE_STRING) {
    return String(value);
  }

  if (type === Const.TYPE_NUMBER) {
    return Number(value);
  }

  if (type === Const.TYPE_BOOLEAN) {
    if (typeof value === 'boolean') {
      return value;
    }

    return value === 'true';
  }

  if (type === Const.TYPE_DATE) {
    return new Date(value);
  }

  return value;
};

var type = /*#__PURE__*/Object.freeze({
  __proto__: null,
  typeConvert: typeConvert
});

var dataOperator = _objectSpread2(_objectSpread2(_objectSpread2(_objectSpread2(_objectSpread2(_objectSpread2({}, rows), selection), expand), mutate), sort$1), type);

const SelectionContext = /*#__PURE__*/React__default['default'].createContext();

class SelectionProvider extends React__default['default'].Component {
  constructor(props) {
    var _props$selectRow$sele, _props$selectRow;

    super(props); // this.selected = props.selectRow.selected || [];

    this.handleRowSelect = (rowKey, checked, rowIndex, e) => {
      const {
        data,
        keyField,
        selectRow: {
          mode,
          onSelect
        }
      } = this.props;
      const {
        selected
      } = this.state;
      const {
        ROW_SELECT_SINGLE
      } = Const; // let currSelected = [...this.selected];

      let currSelected = [...selected];
      let result = true;

      if (onSelect) {
        const row = dataOperator.getRowByRowId(data, keyField, rowKey);
        result = onSelect(row, checked, rowIndex, e);
      }

      if (result === true || result === undefined) {
        if (mode === ROW_SELECT_SINGLE) {
          // when select mode is radio
          currSelected = [rowKey];
        } else if (checked) {
          // when select mode is checkbox
          currSelected.push(rowKey);
        } else {
          currSelected = currSelected.filter(value => value !== rowKey);
        }
      } // this.selected = currSelected;


      this.setState({
        selected: currSelected
      }, () => {
        this.forceUpdate();
      });
    };

    this.handleAllRowsSelect = (e, isUnSelect) => {
      const {
        data,
        keyField,
        selectRow: {
          onSelectAll,
          nonSelectable
        }
      } = this.props; // const { selected } = this;

      const {
        selected
      } = this.state;
      let currSelected;

      if (!isUnSelect) {
        currSelected = selected.concat(dataOperator.selectableKeys(data, keyField, nonSelectable));
      } else {
        currSelected = selected.filter(s => typeof data.find(d => _.get(d, keyField) === s) === 'undefined');
      }

      let result;

      if (onSelectAll) {
        result = onSelectAll(!isUnSelect, dataOperator.getSelectedRows(data, keyField, isUnSelect ? selected : currSelected), e);

        if (Array.isArray(result)) {
          currSelected = result;
        }
      } // this.selected = currSelected;


      this.setState({
        selected: currSelected
      }, () => {
        this.forceUpdate();
      });
    };

    this.state = {
      selected: (_props$selectRow$sele = props === null || props === void 0 ? void 0 : (_props$selectRow = props.selectRow) === null || _props$selectRow === void 0 ? void 0 : _props$selectRow.selected) !== null && _props$selectRow$sele !== void 0 ? _props$selectRow$sele : []
    };
  }

  static getDerivedStateFromProps(nextProps) {
    var _nextProps$selectRow$, _nextProps$selectRow;

    return {
      selected: (_nextProps$selectRow$ = nextProps === null || nextProps === void 0 ? void 0 : (_nextProps$selectRow = nextProps.selectRow) === null || _nextProps$selectRow === void 0 ? void 0 : _nextProps$selectRow.selected) !== null && _nextProps$selectRow$ !== void 0 ? _nextProps$selectRow$ : []
    };
  } // UNSAFE_componentWillReceiveProps(nextProps) {


  componentDidUpdate(prevProps) {
    if (this.props.selectRow !== (prevProps === null || prevProps === void 0 ? void 0 : prevProps.selectRow)) {
      // this.selected = this.props.selectRow.selected || this.selected;
      // eslint-disable-next-line react/no-did-update-set-state
      this.setState(({
        selected
      }) => {
        var _this$props$selectRow, _this$props, _this$props$selectRow2;

        return {
          selected: (_this$props$selectRow = (_this$props = this.props) === null || _this$props === void 0 ? void 0 : (_this$props$selectRow2 = _this$props.selectRow) === null || _this$props$selectRow2 === void 0 ? void 0 : _this$props$selectRow2.selected) !== null && _this$props$selectRow !== void 0 ? _this$props$selectRow : selected
        };
      });
    }
  } // exposed API


  getSelected() {
    // return this.selected;
    // eslint-disable-next-line react/destructuring-assignment
    return this.state.selected;
  }

  render() {
    const {
      data,
      keyField,
      selectRow,
      children
    } = this.props;
    const {
      selected
    } = this.state;
    const {
      allRowsSelected,
      allRowsNotSelected
    } = getSelectionSummary(data, keyField, selected);
    let checkedStatus; // checkbox status depending on selected rows counts

    if (allRowsSelected) checkedStatus = Const.CHECKBOX_STATUS_CHECKED;else if (allRowsNotSelected) checkedStatus = Const.CHECKBOX_STATUS_UNCHECKED;else checkedStatus = Const.CHECKBOX_STATUS_INDETERMINATE;
    return /*#__PURE__*/React__default['default'].createElement(SelectionContext.Provider, {
      value: _objectSpread2(_objectSpread2({}, selectRow), {}, {
        // selected: this.selected,
        selected,
        onRowSelect: this.handleRowSelect,
        onAllRowsSelect: this.handleAllRowsSelect,
        allRowsSelected,
        allRowsNotSelected,
        checkedStatus
      })
    }, children);
  }

}

SelectionProvider.propTypes = {
  children: PropTypes__default['default'].node.isRequired,
  data: PropTypes__default['default'].array.isRequired,
  keyField: PropTypes__default['default'].string.isRequired
};
var SelectionContext$1 = {
  Provider: SelectionProvider,
  Consumer: SelectionContext.Consumer
};

/* eslint-disable react/jsx-props-no-spreading */
var withHeaderSelection = (Component => () => /*#__PURE__*/React__default['default'].createElement(SelectionContext$1.Consumer, null, selectRow => /*#__PURE__*/React__default['default'].createElement(Component, selectRow)));

const RowExpandContext = /*#__PURE__*/React__default['default'].createContext();

class RowExpandProvider extends React__default['default'].Component {
  constructor(...args) {
    super(...args);
    this.state = {
      // eslint-disable-next-line react/destructuring-assignment
      expanded: this.props.expandRow.expanded || [],
      // eslint-disable-next-line react/destructuring-assignment
      isClosing: this.props.expandRow.isClosing || []
    };

    this.onClosed = closedRow => {
      // eslint-disable-next-line max-len
      this.setState(({
        isClosing
      }) => ({
        isClosing: isClosing.filter(value => value !== closedRow)
      }));
    };

    this.handleRowExpand = (rowKey, expanded, rowIndex, e) => {
      const {
        expanded: expandedState,
        isClosing: isClosingState
      } = this.state;
      const {
        data,
        keyField,
        expandRow: {
          onExpand,
          onlyOneExpanding,
          nonExpandable
        }
      } = this.props;

      if (nonExpandable && _.contains(nonExpandable, rowKey)) {
        return;
      }

      let currExpanded = [...expandedState];
      let isClosing = [...isClosingState];

      if (expanded) {
        if (onlyOneExpanding) {
          isClosing = isClosing.concat(currExpanded);
          currExpanded = [rowKey];
        } else currExpanded.push(rowKey);
      } else {
        isClosing.push(rowKey);
        currExpanded = currExpanded.filter(value => value !== rowKey);
      }

      if (onExpand) {
        const row = dataOperator.getRowByRowId(data, keyField, rowKey);
        onExpand(row, expanded, rowIndex, e);
      }

      this.setState(() => ({
        expanded: currExpanded,
        isClosing
      }));
    };

    this.handleAllRowExpand = (e, expandAll) => {
      const {
        data,
        keyField,
        expandRow: {
          onExpandAll,
          nonExpandable
        }
      } = this.props;
      const {
        expanded
      } = this.state;
      let currExpanded;

      if (expandAll) {
        currExpanded = expanded.concat(dataOperator.expandableKeys(data, keyField, nonExpandable));
      } else {
        currExpanded = expanded.filter(s => typeof data.find(d => _.get(d, keyField) === s) === 'undefined');
      }

      if (onExpandAll) {
        onExpandAll(expandAll, dataOperator.getExpandedRows(data, keyField, currExpanded), e);
      }

      this.setState(() => ({
        expanded: currExpanded
      }));
    };
  }

  static getDerivedStateFromProps(props) {
    var _props$expandRow$expa, _props$expandRow, _props$expandRow$isCl, _props$expandRow2;

    return {
      expanded: (_props$expandRow$expa = props === null || props === void 0 ? void 0 : (_props$expandRow = props.expandRow) === null || _props$expandRow === void 0 ? void 0 : _props$expandRow.expanded) !== null && _props$expandRow$expa !== void 0 ? _props$expandRow$expa : [],
      isClosing: (_props$expandRow$isCl = props === null || props === void 0 ? void 0 : (_props$expandRow2 = props.expandRow) === null || _props$expandRow2 === void 0 ? void 0 : _props$expandRow2.isClosing) !== null && _props$expandRow$isCl !== void 0 ? _props$expandRow$isCl : []
    };
  }

  componentDidUpdate() {
    const {
      expanded
    } = this.state;

    if (this.props.expandRow) {
      var _this$props$expandRow, _this$props, _this$props$expandRow2;

      let nextExpanded = [...((_this$props$expandRow = (_this$props = this.props) === null || _this$props === void 0 ? void 0 : (_this$props$expandRow2 = _this$props.expandRow) === null || _this$props$expandRow2 === void 0 ? void 0 : _this$props$expandRow2.expanded) !== null && _this$props$expandRow !== void 0 ? _this$props$expandRow : expanded)];
      const {
        nonExpandable = []
      } = this.props.expandRow;
      nextExpanded = nextExpanded.filter(rowId => !_.contains(nonExpandable, rowId));
      const isClosing = expanded.reduce((acc, cur) => {
        if (!_.contains(nextExpanded, cur)) {
          acc.push(cur);
        }

        return acc;
      }, []);
      this.setState(() => ({
        expanded: nextExpanded,
        isClosing
      }));
    } else {
      this.setState(() => ({
        expanded
      }));
    }
  }

  render() {
    const {
      data,
      keyField,
      expandRow,
      children
    } = this.props;
    const {
      expanded,
      isClosing
    } = this.state;
    return /*#__PURE__*/React__default['default'].createElement(RowExpandContext.Provider, {
      value: _objectSpread2(_objectSpread2({}, expandRow), {}, {
        nonExpandable: expandRow.nonExpandable,
        expanded,
        isClosing,
        onClosed: this.onClosed,
        isAnyExpands: dataOperator.isAnyExpands(data, keyField, expanded),
        onRowExpand: this.handleRowExpand,
        onAllRowExpand: this.handleAllRowExpand
      })
    }, children);
  }

}

RowExpandProvider.propTypes = {
  children: PropTypes__default['default'].node.isRequired,
  data: PropTypes__default['default'].array.isRequired,
  keyField: PropTypes__default['default'].string.isRequired
};
var RowExpandContext$1 = {
  Provider: RowExpandProvider,
  Consumer: RowExpandContext.Consumer
};

/* eslint-disable react/jsx-props-no-spreading */
var withHeaderExpansion = (Component => () => /*#__PURE__*/React__default['default'].createElement(RowExpandContext$1.Consumer, null, expandRow => /*#__PURE__*/React__default['default'].createElement(Component, expandRow)));

/* eslint react/require-default-props: 0 */

const Header = props => {
  const {
    className,
    columns,
    onSort,
    onFilter,
    sortField,
    sortOrder,
    selectRow,
    expandRow,
    currFilters,
    onExternalFilter,
    filterPosition,
    globalSortCaret,
    wrapperClasses
  } = props;

  let SelectionHeaderCellComp = () => null;

  let ExpansionHeaderCellComp = () => null;

  if (expandRow.showExpandColumn) {
    ExpansionHeaderCellComp = withHeaderExpansion(ExpansionHeaderCell);
  }

  if (selectRow) {
    SelectionHeaderCellComp = withHeaderSelection(SelectionHeaderCell);
  }

  const isRenderFunctionColumnInLeft = (position = Const.INDICATOR_POSITION_LEFT) => position === Const.INDICATOR_POSITION_LEFT;

  const childrens = [columns.map((column, i) => {
    const currSort = column.dataField === sortField;
    const isLastSorting = column.dataField === sortField;
    return /*#__PURE__*/React__default['default'].createElement(HeaderCell, {
      index: i,
      key: column.dataField,
      column: column,
      onSort: onSort,
      sorting: currSort,
      sortOrder: sortOrder,
      globalSortCaret: globalSortCaret,
      isLastSorting: isLastSorting,
      onFilter: onFilter,
      currFilters: currFilters,
      onExternalFilter: onExternalFilter,
      filterPosition: filterPosition
    });
  })];

  if (!selectRow.hideSelectColumn) {
    if (isRenderFunctionColumnInLeft(selectRow.selectColumnPosition)) {
      childrens.unshift( /*#__PURE__*/React__default['default'].createElement(SelectionHeaderCellComp, {
        key: "selection"
      }));
    } else {
      childrens.push( /*#__PURE__*/React__default['default'].createElement(SelectionHeaderCellComp, {
        key: "selection"
      }));
    }
  }

  if (expandRow.showExpandColumn) {
    if (isRenderFunctionColumnInLeft(expandRow.expandColumnPosition)) {
      childrens.unshift( /*#__PURE__*/React__default['default'].createElement(ExpansionHeaderCellComp, {
        key: "expansion"
      }));
    } else {
      childrens.push( /*#__PURE__*/React__default['default'].createElement(ExpansionHeaderCellComp, {
        key: "expansion"
      }));
    }
  }

  return /*#__PURE__*/React__default['default'].createElement("thead", {
    className: wrapperClasses
  }, /*#__PURE__*/React__default['default'].createElement("tr", {
    className: className
  }, childrens));
};

Header.propTypes = {
  columns: PropTypes__default['default'].array.isRequired,
  onSort: PropTypes__default['default'].func,
  onFilter: PropTypes__default['default'].func,
  sortField: PropTypes__default['default'].string,
  sortOrder: PropTypes__default['default'].string,
  selectRow: PropTypes__default['default'].object,
  currFilters: PropTypes__default['default'].object,
  onExternalFilter: PropTypes__default['default'].func,
  globalSortCaret: PropTypes__default['default'].func,
  className: PropTypes__default['default'].string,
  wrapperClasses: PropTypes__default['default'].string,
  expandRow: PropTypes__default['default'].object,
  filterPosition: PropTypes__default['default'].oneOf([Const.FILTERS_POSITION_TOP, Const.FILTERS_POSITION_INLINE, Const.FILTERS_POSITION_BOTTOM])
};

const FiltersCell = props => {
  const {
    index,
    column,
    onExternalFilter,
    currFilters,
    onFilter
  } = props;
  const {
    filterRenderer,
    filter
  } = column;
  let filterElm;
  const cellAttrs = {};
  const cellStyle = {};
  cellAttrs.style = cellStyle;

  if (column.headerAlign) {
    cellStyle.textAlign = _.isFunction(column.headerAlign) ? column.headerAlign(column, index) : column.headerAlign;
  }

  if (column.filterRenderer) {
    const onCustomFilter = onExternalFilter(column, filter.props.type);
    filterElm = filterRenderer(onCustomFilter, column);
  } else if (filter) {
    filterElm = /*#__PURE__*/React__default['default'].createElement(filter.Filter, _extends({}, filter.props, {
      filterState: currFilters[column.dataField],
      onFilter: onFilter,
      column: column
    }));
  }

  return /*#__PURE__*/React__default['default'].createElement('th', cellAttrs, filterElm);
};

FiltersCell.propTypes = {
  index: PropTypes__default['default'].number.isRequired,
  column: PropTypes__default['default'].object.isRequired,
  currFilters: PropTypes__default['default'].object.isRequired,
  onFilter: PropTypes__default['default'].func,
  onExternalFilter: PropTypes__default['default'].func
};
FiltersCell.defaultProps = {
  onFilter: () => {},
  onExternalFilter: () => {}
};

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};
  var target = _objectWithoutPropertiesLoose(source, excluded);
  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

const RowTemplate = props => {
  const {
    renderContent,
    selectRow,
    expandRow,
    cellEl
  } = props,
        rest = _objectWithoutProperties(props, ["renderContent", "selectRow", "expandRow", "cellEl"]);

  const isRenderFunctionColumnInLeft = (position = Const.INDICATOR_POSITION_LEFT) => position === Const.INDICATOR_POSITION_LEFT;

  const childrens = renderContent() || [];

  if (selectRow && selectRow.hideSelectColumn !== true) {
    if (isRenderFunctionColumnInLeft(selectRow.selectColumnPosition)) {
      childrens.unshift( /*#__PURE__*/React__default['default'].createElement(cellEl, {
        key: 'selection'
      }));
    } else {
      childrens.push( /*#__PURE__*/React__default['default'].createElement(cellEl, {
        key: 'selection'
      }));
    }
  }

  if (expandRow.showExpandColumn) {
    if (isRenderFunctionColumnInLeft(expandRow.expandColumnPosition)) {
      childrens.unshift( /*#__PURE__*/React__default['default'].createElement(cellEl, {
        key: 'expansion'
      }));
    } else {
      childrens.push( /*#__PURE__*/React__default['default'].createElement(cellEl, {
        key: 'expansion'
      }));
    }
  }

  return /*#__PURE__*/React__default['default'].createElement("tr", rest, childrens);
};

RowTemplate.propTypes = {
  renderContent: PropTypes__default['default'].func.isRequired,
  cellEl: PropTypes__default['default'].string.isRequired,
  selectRow: PropTypes__default['default'].object,
  expandRow: PropTypes__default['default'].object
};

/* eslint react/require-default-props: 0 */

const Filters = props => {
  const {
    columns,
    onFilter,
    currFilters,
    filterPosition,
    onExternalFilter,
    className,
    selectRow,
    expandRow
  } = props;

  function renderContent() {
    const filterColumns = [];
    columns.forEach((column, i) => {
      filterColumns.push( /*#__PURE__*/React__default['default'].createElement(FiltersCell, {
        index: i,
        key: column.dataField,
        column: column,
        currFilters: currFilters,
        onExternalFilter: onExternalFilter,
        onFilter: onFilter
      }));

      if (column.filterRenderer || column.filter) ;
    });
    return filterColumns;
  }

  return /*#__PURE__*/React__default['default'].createElement("tbody", {
    className: className,
    style: {
      display: filterPosition === Const.FILTERS_POSITION_TOP ? 'table-header-group' : 'table-footer-group'
    }
  }, /*#__PURE__*/React__default['default'].createElement(RowTemplate, {
    renderContent: renderContent,
    selectRow: selectRow,
    expandRow: expandRow,
    cellEl: "td"
  }));
};

Filters.propTypes = {
  columns: PropTypes__default['default'].array.isRequired,
  onFilter: PropTypes__default['default'].func,
  filterPosition: PropTypes__default['default'].oneOf([Const.FILTERS_POSITION_TOP, Const.FILTERS_POSITION_INLINE, Const.FILTERS_POSITION_BOTTOM]),
  currFilters: PropTypes__default['default'].object,
  onExternalFilter: PropTypes__default['default'].func,
  className: PropTypes__default['default'].string,
  selectRow: PropTypes__default['default'].object,
  expandRow: PropTypes__default['default'].object
};
Filters.defaultProps = {
  filterPosition: Const.FILTERS_POSITION_TOP
};

/* eslint react/require-default-props: 0 */

const Caption = props => {
  if (!props.children) return null;
  const caption = props.bootstrap4 ? /*#__PURE__*/React__default['default'].createElement("caption", {
    style: {
      captionSide: 'top'
    }
  }, props.children) : /*#__PURE__*/React__default['default'].createElement("caption", null, props.children);
  return caption;
};

Caption.propTypes = {
  children: PropTypes__default['default'].oneOfType([PropTypes__default['default'].node, PropTypes__default['default'].string]),
  bootstrap4: PropTypes__default['default'].bool
};

class Cell extends eventDelegater(React.Component) {
  constructor(...args) {
    super(...args);

    this.createHandleEditingCell = originFunc => e => {
      const {
        onStart,
        rowIndex,
        columnIndex,
        clickToEdit,
        dbclickToEdit
      } = this.props;

      if ((clickToEdit || dbclickToEdit) && _.isFunction(originFunc)) {
        originFunc(e);
      }

      if (onStart) {
        onStart(rowIndex, columnIndex);
      }
    };
  }

  shouldComponentUpdate(nextProps) {
    let shouldUpdate = false;

    if (nextProps.column.isDummyField) {
      shouldUpdate = !_.isEqual(this.props.row, nextProps.row);
    } else {
      shouldUpdate = _.get(this.props.row, this.props.column.dataField) !== _.get(nextProps.row, nextProps.column.dataField);
    }

    if (shouldUpdate) return true; // if (nextProps.formatter)

    shouldUpdate = (nextProps.column.formatter ? !_.isEqual(this.props.row, nextProps.row) : false) || this.props.column.hidden !== nextProps.column.hidden || this.props.column.isDummyField !== nextProps.column.isDummyField || this.props.rowIndex !== nextProps.rowIndex || this.props.columnIndex !== nextProps.columnIndex || this.props.className !== nextProps.className || this.props.title !== nextProps.title || this.props.editable !== nextProps.editable || this.props.clickToEdit !== nextProps.clickToEdit || this.props.dbclickToEdit !== nextProps.dbclickToEdit || !_.isEqual(this.props.style, nextProps.style) || !_.isEqual(this.props.column.formatExtraData, nextProps.column.formatExtraData) || !_.isEqual(this.props.column.events, nextProps.column.events) || !_.isEqual(this.props.column.attrs, nextProps.column.attrs) || this.props.tabIndex !== nextProps.tabIndex;
    return shouldUpdate;
  }

  render() {
    const _this$props = this.props,
          {
      row,
      rowIndex,
      column,
      columnIndex,
      onStart,
      editable,
      clickToEdit,
      dbclickToEdit
    } = _this$props,
          rest = _objectWithoutProperties(_this$props, ["row", "rowIndex", "column", "columnIndex", "onStart", "editable", "clickToEdit", "dbclickToEdit"]);

    const {
      dataField,
      formatter,
      formatExtraData
    } = column;
    const attrs = this.delegate(_objectSpread2({}, rest));
    let content = column.isDummyField ? null : _.get(row, dataField);

    if (formatter) {
      content = column.formatter(content, row, rowIndex, formatExtraData);
    }

    if (clickToEdit && editable) {
      attrs.onClick = this.createHandleEditingCell(attrs.onClick);
    } else if (dbclickToEdit && editable) {
      attrs.onDoubleClick = this.createHandleEditingCell(attrs.onDoubleClick);
    }

    return /*#__PURE__*/React__default['default'].createElement("td", attrs, typeof content === 'boolean' ? `${content}` : content);
  }

}

Cell.propTypes = {
  row: PropTypes__default['default'].object.isRequired,
  rowIndex: PropTypes__default['default'].number.isRequired,
  column: PropTypes__default['default'].object.isRequired,
  columnIndex: PropTypes__default['default'].number.isRequired
};

class RowPureContent extends React__default['default'].Component {
  shouldComponentUpdate(nextProps) {
    if (typeof nextProps.shouldUpdate !== 'undefined') {
      return nextProps.shouldUpdate;
    }

    return true;
  }

  render() {
    const {
      row,
      keyField,
      columns,
      rowIndex,
      editable,
      editingRowIdx,
      editingColIdx,
      onStart,
      clickToEdit,
      dbclickToEdit,
      EditingCellComponent,
      tabIndexStart
    } = this.props;
    let tabIndex = tabIndexStart;
    return columns.map((column, index) => {
      const {
        dataField
      } = column;

      const content = _.get(row, dataField);

      if (rowIndex === editingRowIdx && index === editingColIdx) {
        return /*#__PURE__*/React__default['default'].createElement(EditingCellComponent, {
          key: `${content}-${index}-editing`,
          row: row,
          rowIndex: rowIndex,
          column: column,
          columnIndex: index
        });
      } // render cell


      let cellTitle;
      let cellStyle = {};

      let cellAttrs = _objectSpread2({}, _.isFunction(column.attrs) ? column.attrs(content, row, rowIndex, index) : column.attrs);

      if (column.events) {
        const events = _objectSpread2({}, column.events);

        Object.keys(_objectSpread2({}, column.events)).forEach(key => {
          const originFn = events[key];

          events[key] = (...rest) => originFn(...rest, row, rowIndex);
        });
        cellAttrs = _objectSpread2(_objectSpread2({}, cellAttrs), events);
      }

      const cellClasses = _.isFunction(column.classes) ? column.classes(content, row, rowIndex, index) : column.classes;

      if (column.style) {
        cellStyle = _.isFunction(column.style) ? column.style(content, row, rowIndex, index) : column.style;
        cellStyle = _objectSpread2({}, cellStyle) || {};
      }

      if (column.title) {
        cellTitle = _.isFunction(column.title) ? column.title(content, row, rowIndex, index) : content;
        cellAttrs.title = cellTitle;
      }

      if (column.align) {
        cellStyle.textAlign = _.isFunction(column.align) ? column.align(content, row, rowIndex, index) : column.align;
      }

      if (cellClasses) cellAttrs.className = cellClasses;
      if (!_.isEmptyObject(cellStyle)) cellAttrs.style = cellStyle;
      let editableCell = _.isDefined(column.editable) ? column.editable : true;
      if (column.dataField === keyField || !editable) editableCell = false;

      if (_.isFunction(column.editable)) {
        editableCell = column.editable(content, row, rowIndex, index);
      }

      if (tabIndexStart !== -1) {
        cellAttrs.tabIndex = tabIndex++;
      }

      return /*#__PURE__*/React__default['default'].createElement(Cell, _extends({
        key: `${content}-${index}`,
        row: row,
        editable: editableCell,
        rowIndex: rowIndex,
        columnIndex: index,
        column: column,
        onStart: onStart,
        clickToEdit: clickToEdit,
        dbclickToEdit: dbclickToEdit
      }, cellAttrs));
    });
  }

}

const events$1 = ['onClick', 'onDoubleClick', 'onMouseEnter', 'onMouseLeave', 'onContextMenu', 'onAuxClick'];
var eventDelegater$1 = (ExtendBase => class RowEventDelegater extends ExtendBase {
  constructor(props) {
    super(props);
    this.clickNum = 0;
    this.createDefaultEventHandler = this.createDefaultEventHandler.bind(this);
    this.createClickEventHandler = this.createClickEventHandler.bind(this);
  }

  createClickEventHandler(cb) {
    return e => {
      const {
        row,
        selected,
        keyField,
        selectable,
        expandable,
        rowIndex,
        expanded,
        expandRow,
        selectRow,
        DELAY_FOR_DBCLICK
      } = this.props;

      const clickFn = () => {
        if (cb) {
          cb(e, row, rowIndex);
        }

        const key = _.get(row, keyField);

        if (expandRow && expandable && !expandRow.expandByColumnOnly) {
          if (selectRow.mode !== Const.ROW_SELECT_DISABLED && selectRow.clickToExpand || selectRow.mode === Const.ROW_SELECT_DISABLED) {
            expandRow.onRowExpand(key, !expanded, rowIndex, e);
          }
        }

        if (selectRow.clickToSelect && selectable) {
          selectRow.onRowSelect(key, !selected, rowIndex, e);
        }
      };

      if (DELAY_FOR_DBCLICK) {
        this.clickNum += 1;

        _.debounce(() => {
          if (this.clickNum === 1) {
            clickFn();
          }

          this.clickNum = 0;
        }, DELAY_FOR_DBCLICK)();
      } else {
        clickFn();
      }
    };
  }

  createDefaultEventHandler(cb) {
    return e => {
      const {
        row,
        rowIndex
      } = this.props;
      cb(e, row, rowIndex);
    };
  }

  delegate(attrs = {}) {
    const newAttrs = _objectSpread2({}, attrs);

    Object.keys(attrs).forEach(attr => {
      if (_.contains(events$1, attr)) {
        newAttrs[attr] = this.createDefaultEventHandler(attrs[attr]);
      }
    });
    return newAttrs;
  }

});

/* eslint react/prop-types: 0 */
var shouldUpdater = (ExtendBase => class RowShouldUpdater extends ExtendBase {
  shouldUpdateByCellEditing(nextProps) {
    if (!(this.props.clickToEdit || this.props.dbclickToEdit)) return false;
    return nextProps.editingRowIdx === nextProps.rowIndex || this.props.editingRowIdx === nextProps.rowIndex && nextProps.editingRowIdx === null || this.props.editingRowIdx === nextProps.rowIndex;
  }

  shouldUpdatedBySelfProps(nextProps) {
    return this.props.className !== nextProps.className || !_.isEqual(this.props.style, nextProps.style) || !_.isEqual(this.props.attrs, nextProps.attrs);
  } // Only use for simple-row


  shouldUpdateByColumnsForSimpleCheck(nextProps) {
    if (this.props.columns.length !== nextProps.columns.length) {
      return true;
    }

    for (let i = 0; i < this.props.columns.length; i += 1) {
      if (!_.isEqual(this.props.columns[i], nextProps.columns[i])) {
        return true;
      }
    }

    return false;
  }

  shouldUpdatedByNormalProps(nextProps) {
    const shouldUpdate = this.props.rowIndex !== nextProps.rowIndex || this.props.editable !== nextProps.editable || !_.isEqual(this.props.row, nextProps.row) || this.props.columns.length !== nextProps.columns.length;
    return shouldUpdate;
  }

  shouldUpdateChild(nextProps) {
    return this.shouldUpdateByCellEditing(nextProps) || this.shouldUpdatedByNormalProps(nextProps);
  }

  shouldRowContentUpdate(nextProps) {
    return this.shouldUpdateChild(nextProps) || this.shouldUpdateByColumnsForSimpleCheck(nextProps);
  }

});

class SimpleRow extends shouldUpdater(eventDelegater$1(React.Component)) {
  constructor(props) {
    super(props);
    this.shouldUpdateRowContent = false;
  }

  shouldComponentUpdate(nextProps) {
    this.shouldUpdateRowContent = false;
    this.shouldUpdateRowContent = this.shouldRowContentUpdate(nextProps);
    if (this.shouldUpdateRowContent) return true;
    return this.shouldUpdatedBySelfProps(nextProps);
  }

  render() {
    const _this$props = this.props,
          {
      className,
      style,
      attrs,
      visibleColumnSize,
      tabIndexCell
    } = _this$props,
          rest = _objectWithoutProperties(_this$props, ["className", "style", "attrs", "visibleColumnSize", "tabIndexCell"]);

    const trAttrs = this.delegate(attrs);
    const tabIndexStart = this.props.rowIndex * visibleColumnSize + 1;
    return /*#__PURE__*/React__default['default'].createElement("tr", _extends({
      style: style,
      className: className
    }, trAttrs), /*#__PURE__*/React__default['default'].createElement(RowPureContent, _extends({
      shouldUpdate: this.shouldUpdateRowContent,
      tabIndexStart: tabIndexCell ? tabIndexStart : -1
    }, rest)));
  }

}

SimpleRow.propTypes = {
  row: PropTypes__default['default'].object.isRequired,
  rowIndex: PropTypes__default['default'].number.isRequired,
  columns: PropTypes__default['default'].array.isRequired,
  style: PropTypes__default['default'].object,
  className: PropTypes__default['default'].string,
  attrs: PropTypes__default['default'].object
};
SimpleRow.defaultProps = {
  editable: true,
  style: {},
  className: null,
  attrs: {}
};

class ExpandCell extends React.Component {
  constructor(...args) {
    super(...args);

    this.handleClick = e => {
      const {
        rowKey,
        expanded,
        onRowExpand,
        rowIndex
      } = this.props;
      e.stopPropagation();
      onRowExpand(rowKey, !expanded, rowIndex, e);
    };
  }

  shouldComponentUpdate(nextProps) {
    const {
      rowIndex,
      expanded,
      rowKey,
      tabIndex
    } = this.props;
    const shouldUpdate = rowIndex !== nextProps.rowIndex || expanded !== nextProps.expanded || rowKey !== nextProps.rowKey || tabIndex !== nextProps.tabIndex;
    return shouldUpdate;
  }

  render() {
    const {
      expanded,
      expandable,
      expandColumnRenderer,
      tabIndex,
      rowKey
    } = this.props;
    const attrs = {};
    if (tabIndex !== -1) attrs.tabIndex = tabIndex;
    return /*#__PURE__*/React__default['default'].createElement("td", _extends({
      className: "expand-cell",
      onClick: this.handleClick
    }, attrs), expandColumnRenderer ? expandColumnRenderer({
      expandable,
      expanded,
      rowKey
    }) : expandable ? expanded ? '(-)' : '(+)' : '');
  }

}

ExpandCell.propTypes = {
  rowKey: PropTypes__default['default'].any,
  expanded: PropTypes__default['default'].bool.isRequired,
  expandable: PropTypes__default['default'].bool.isRequired,
  onRowExpand: PropTypes__default['default'].func.isRequired,
  expandColumnRenderer: PropTypes__default['default'].func,
  rowIndex: PropTypes__default['default'].number,
  tabIndex: PropTypes__default['default'].number
};

class SelectionCell extends React.Component {
  constructor(...args) {
    super(...args);

    this.handleClick = e => {
      const {
        mode: inputType,
        rowKey,
        selected,
        onRowSelect,
        disabled,
        rowIndex
      } = this.props;
      e.stopPropagation();
      if (disabled) return;
      const checked = inputType === Const.ROW_SELECT_SINGLE ? true : !selected;
      onRowSelect(rowKey, checked, rowIndex, e);
    };
  }

  shouldComponentUpdate(nextProps) {
    const {
      rowIndex,
      selected,
      disabled,
      rowKey,
      tabIndex,
      selectColumnStyle
    } = this.props;
    const shouldUpdate = rowIndex !== nextProps.rowIndex || selected !== nextProps.selected || disabled !== nextProps.disabled || rowKey !== nextProps.rowKey || tabIndex !== nextProps.tabIndex || selectColumnStyle !== nextProps.selectColumnStyle;
    return shouldUpdate;
  }

  render() {
    const {
      rowKey,
      mode: inputType,
      selected,
      disabled,
      tabIndex,
      rowIndex,
      selectionRenderer,
      selectColumnStyle
    } = this.props;
    const attrs = {};
    if (tabIndex !== -1) attrs.tabIndex = tabIndex;
    attrs.style = _.isFunction(selectColumnStyle) ? selectColumnStyle({
      checked: selected,
      disabled,
      rowIndex,
      rowKey
    }) : selectColumnStyle;
    return /*#__PURE__*/React__default['default'].createElement(BootstrapContext.Consumer, null, ({
      bootstrap4
    }) => /*#__PURE__*/React__default['default'].createElement("td", _extends({
      className: "selection-cell",
      onClick: this.handleClick
    }, attrs), selectionRenderer ? selectionRenderer({
      mode: inputType,
      checked: selected,
      disabled,
      rowIndex,
      rowKey
    }) : /*#__PURE__*/React__default['default'].createElement("input", {
      type: inputType,
      checked: selected,
      disabled: disabled,
      className: bootstrap4 ? 'selection-input-4' : '',
      onChange: () => {}
    })));
  }

}

SelectionCell.propTypes = {
  mode: PropTypes__default['default'].string.isRequired,
  rowKey: PropTypes__default['default'].any,
  selected: PropTypes__default['default'].bool,
  onRowSelect: PropTypes__default['default'].func,
  disabled: PropTypes__default['default'].bool,
  rowIndex: PropTypes__default['default'].number,
  tabIndex: PropTypes__default['default'].number,
  clickToSelect: PropTypes__default['default'].bool,
  selectionRenderer: PropTypes__default['default'].func,
  selectColumnStyle: PropTypes__default['default'].oneOfType([PropTypes__default['default'].object, PropTypes__default['default'].func])
};

class RowAggregator extends shouldUpdater(eventDelegater$1(React__default['default'].Component)) {
  constructor(props) {
    super(props);
    this.clickNum = 0;
    this.shouldUpdateRowContent = false;
    this.createClickEventHandler = this.createClickEventHandler.bind(this);
  }

  shouldComponentUpdate(nextProps) {
    if (this.props.selected !== nextProps.selected || this.props.expanded !== nextProps.expanded || this.props.expandable !== nextProps.expandable || this.props.selectable !== nextProps.selectable || this.props.selectRow.hideSelectColumn !== nextProps.selectRow.hideSelectColumn || this.shouldUpdatedBySelfProps(nextProps)) {
      this.shouldUpdateRowContent = this.shouldRowContentUpdate(nextProps);
      return true;
    }

    this.shouldUpdateRowContent = this.shouldRowContentUpdate(nextProps);
    return this.shouldUpdateRowContent;
  }

  isRenderFunctionColumnInLeft(position = Const.INDICATOR_POSITION_LEFT) {
    return position === Const.INDICATOR_POSITION_LEFT;
  }

  render() {
    const _this$props = this.props,
          {
      row,
      columns,
      keyField,
      rowIndex,
      style,
      className,
      attrs,
      selectRow,
      expandRow,
      expanded,
      expandable,
      selected,
      selectable,
      visibleColumnSize,
      tabIndexCell
    } = _this$props,
          rest = _objectWithoutProperties(_this$props, ["row", "columns", "keyField", "rowIndex", "style", "className", "attrs", "selectRow", "expandRow", "expanded", "expandable", "selected", "selectable", "visibleColumnSize", "tabIndexCell"]);

    const key = _.get(row, keyField);

    const {
      hideSelectColumn,
      selectColumnPosition,
      clickToSelect
    } = selectRow;
    const {
      showExpandColumn,
      expandColumnPosition
    } = expandRow;
    const newAttrs = this.delegate(_objectSpread2({}, attrs));

    if (clickToSelect || !!expandRow.renderer) {
      newAttrs.onClick = this.createClickEventHandler(newAttrs.onClick);
    }

    let tabIndexStart = rowIndex * visibleColumnSize + 1;
    const childrens = [/*#__PURE__*/React__default['default'].createElement(RowPureContent, _extends({
      key: "row",
      row: row,
      columns: columns,
      keyField: keyField,
      rowIndex: rowIndex,
      shouldUpdate: this.shouldUpdateRowContent,
      tabIndexStart: tabIndexCell ? tabIndexStart : -1
    }, rest))];

    if (!hideSelectColumn) {
      const selectCell = /*#__PURE__*/React__default['default'].createElement(SelectionCell, _extends({}, selectRow, {
        key: "selection-cell",
        rowKey: key,
        rowIndex: rowIndex,
        selected: selected,
        disabled: !selectable,
        tabIndex: tabIndexCell ? tabIndexStart++ : -1
      }));

      if (this.isRenderFunctionColumnInLeft(selectColumnPosition)) {
        childrens.unshift(selectCell);
      } else {
        childrens.push(selectCell);
      }
    }

    if (showExpandColumn) {
      const expandCell = /*#__PURE__*/React__default['default'].createElement(ExpandCell, _extends({}, expandRow, {
        key: "expand-cell",
        rowKey: key,
        rowIndex: rowIndex,
        expanded: expanded,
        expandable: expandable,
        tabIndex: tabIndexCell ? tabIndexStart++ : -1
      }));

      if (this.isRenderFunctionColumnInLeft(expandColumnPosition)) {
        childrens.unshift(expandCell);
      } else {
        childrens.push(expandCell);
      }
    }

    return /*#__PURE__*/React__default['default'].createElement("tr", _extends({
      style: style,
      className: className
    }, newAttrs), childrens);
  }

}

RowAggregator.propTypes = {
  attrs: PropTypes__default['default'].object,
  style: PropTypes__default['default'].object
};
RowAggregator.defaultProps = {
  attrs: {},
  style: {}
};

const RowSection = ({
  content,
  colSpan
}) => /*#__PURE__*/React__default['default'].createElement("tr", null, /*#__PURE__*/React__default['default'].createElement("td", {
  "data-toggle": "collapse",
  colSpan: colSpan,
  className: "react-bs-table-no-data"
}, content));

RowSection.propTypes = {
  content: PropTypes__default['default'].any,
  colSpan: PropTypes__default['default'].number
};
RowSection.defaultProps = {
  content: null,
  colSpan: 1
};

var withRowSelection = (Component => {
  const renderWithSelection = (props, selectRow) => {
    const key = props.value;

    const selected = _.contains(selectRow.selected, key);

    const selectable = !selectRow.nonSelectable || !_.contains(selectRow.nonSelectable, key);

    const notSelectable = _.contains(selectRow.nonSelectable, key);

    let {
      style,
      className
    } = props;

    if (selected) {
      const selectedStyle = _.isFunction(selectRow.style) ? selectRow.style(props.row, props.rowIndex) : selectRow.style;
      const selectedClasses = _.isFunction(selectRow.classes) ? selectRow.classes(props.row, props.rowIndex) : selectRow.classes;
      style = _objectSpread2(_objectSpread2({}, style), selectedStyle);
      className = cs__default['default'](className, selectedClasses) || undefined;

      if (selectRow.bgColor) {
        style = style || {};
        style.backgroundColor = _.isFunction(selectRow.bgColor) ? selectRow.bgColor(props.row, props.rowIndex) : selectRow.bgColor;
      }
    }

    if (notSelectable) {
      const notSelectableStyle = _.isFunction(selectRow.nonSelectableStyle) ? selectRow.nonSelectableStyle(props.row, props.rowIndex) : selectRow.nonSelectableStyle;
      const notSelectableClasses = _.isFunction(selectRow.nonSelectableClasses) ? selectRow.nonSelectableClasses(props.row, props.rowIndex) : selectRow.nonSelectableClasses;
      style = _objectSpread2(_objectSpread2({}, style), notSelectableStyle);
      className = cs__default['default'](className, notSelectableClasses) || undefined;
    }

    return /*#__PURE__*/React__default['default'].createElement(Component, _extends({}, props, {
      style: style,
      className: className,
      selectRow: selectRow,
      selected: selected,
      selectable: selectable
    }));
  };

  function withConsumer(props) {
    return /*#__PURE__*/React__default['default'].createElement(SelectionContext$1.Consumer, null, selectRow => renderWithSelection(props, selectRow));
  }

  withConsumer.displayName = 'WithSelectionRowConsumer';
  return withConsumer;
});

const ExpandRow = (_ref) => {
  let {
    children,
    expanded,
    onClosed,
    className
  } = _ref,
      rest = _objectWithoutProperties(_ref, ["children", "expanded", "onClosed", "className"]);

  return /*#__PURE__*/React__default['default'].createElement("tr", null, /*#__PURE__*/React__default['default'].createElement("td", _extends({
    className: cs__default['default']('reset-expansion-style', className)
  }, rest), /*#__PURE__*/React__default['default'].createElement(reactTransitionGroup.CSSTransition, {
    appear: true,
    in: expanded,
    timeout: 400,
    classNames: "row-expand-slide",
    onExited: onClosed
  }, /*#__PURE__*/React__default['default'].createElement("div", null, /*#__PURE__*/React__default['default'].createElement("div", {
    className: "row-expansion-style"
  }, children)))));
};

ExpandRow.propTypes = {
  children: PropTypes__default['default'].node,
  expanded: PropTypes__default['default'].bool,
  onClosed: PropTypes__default['default'].func,
  className: PropTypes__default['default'].string
};
ExpandRow.defaultProps = {
  children: null,
  expanded: false,
  onClosed: null,
  className: ''
};

var withRowExpansion = (Component => {
  const renderWithExpansion = (props, expandRow) => {
    let parentClassName = '';
    let className = '';
    const key = props.value;

    const expanded = _.contains(expandRow.expanded, key);

    const isClosing = _.contains(expandRow.isClosing, key);

    const expandable = !expandRow.nonExpandable || !_.contains(expandRow.nonExpandable, key);

    if (expanded) {
      parentClassName = _.isFunction(expandRow.parentClassName) ? expandRow.parentClassName(expanded, props.row, props.rowIndex) : expandRow.parentClassName || '';
      className = _.isFunction(expandRow.className) ? expandRow.className(expanded, props.row, props.rowIndex) : expandRow.className || '';
    }

    return [/*#__PURE__*/React__default['default'].createElement(Component, _extends({}, props, {
      key: key,
      expanded: expanded,
      expandable: expandable,
      expandRow: _objectSpread2({}, expandRow),
      className: cs__default['default'](props.className, parentClassName)
    })), expanded || isClosing ? /*#__PURE__*/React__default['default'].createElement(ExpandRow, {
      key: `${key}-expanding`,
      colSpan: props.visibleColumnSize,
      expanded: expanded,
      onClosed: () => expandRow.onClosed(key),
      className: className
    }, expandRow.renderer(props.row, props.rowIndex)) : null];
  };

  return props => /*#__PURE__*/React__default['default'].createElement(RowExpandContext$1.Consumer, null, expandRow => renderWithExpansion(props, expandRow));
});

class Body extends React__default['default'].Component {
  constructor(props) {
    super(props);
    const {
      keyField,
      cellEdit,
      selectRow,
      expandRow
    } = props; // Construct Editing Cell Component

    if (cellEdit.createContext) {
      this.EditingCell = cellEdit.createEditingCell(_, cellEdit.options.onStartEdit);
    } // Construct Row Component


    let RowComponent = SimpleRow;
    const selectRowEnabled = selectRow.mode !== Const.ROW_SELECT_DISABLED;
    const expandRowEnabled = !!expandRow.renderer;

    if (expandRowEnabled) {
      RowComponent = withRowExpansion(RowAggregator);
    }

    if (selectRowEnabled) {
      RowComponent = withRowSelection(expandRowEnabled ? RowComponent : RowAggregator);
    }

    if (cellEdit.createContext) {
      RowComponent = cellEdit.withRowLevelCellEdit(RowComponent, selectRowEnabled, keyField, _);
    }

    this.RowComponent = RowComponent;
  }

  render() {
    const {
      columns,
      data,
      tabIndexCell,
      keyField,
      isEmpty,
      noDataIndication,
      visibleColumnSize,
      cellEdit,
      selectRow,
      rowStyle,
      rowClasses,
      rowEvents,
      expandRow,
      className
    } = this.props;
    let content;

    if (isEmpty) {
      const indication = _.isFunction(noDataIndication) ? noDataIndication() : noDataIndication;

      if (!indication) {
        return null;
      }

      content = /*#__PURE__*/React__default['default'].createElement(RowSection, {
        content: indication,
        colSpan: visibleColumnSize
      });
    } else {
      const selectRowEnabled = selectRow.mode !== Const.ROW_SELECT_DISABLED;
      const expandRowEnabled = !!expandRow.renderer;
      const additionalRowProps = {};

      if (cellEdit.createContext) {
        additionalRowProps.EditingCellComponent = this.EditingCell;
      }

      if (selectRowEnabled || expandRowEnabled) {
        additionalRowProps.expandRow = expandRow;
        additionalRowProps.selectRow = selectRow;
      }

      content = data.map((row, index) => {
        const key = _.get(row, keyField);

        const baseRowProps = _objectSpread2({
          key,
          row,
          tabIndexCell,
          columns,
          keyField,
          cellEdit,
          value: key,
          rowIndex: index,
          visibleColumnSize,
          attrs: rowEvents || {}
        }, additionalRowProps);

        baseRowProps.style = _.isFunction(rowStyle) ? rowStyle(row, index) : rowStyle;
        baseRowProps.className = _.isFunction(rowClasses) ? rowClasses(row, index) : rowClasses;
        return /*#__PURE__*/React__default['default'].createElement(this.RowComponent, baseRowProps);
      });
    }

    return /*#__PURE__*/React__default['default'].createElement("tbody", {
      className: className
    }, content);
  }

}

Body.propTypes = {
  keyField: PropTypes__default['default'].string.isRequired,
  data: PropTypes__default['default'].array.isRequired,
  columns: PropTypes__default['default'].array.isRequired,
  selectRow: PropTypes__default['default'].object
};

class FooterCell extends eventDelegater(React__default['default'].Component) {
  render() {
    const {
      index,
      column,
      columnData
    } = this.props;
    const {
      footer,
      footerTitle,
      footerAlign,
      footerFormatter,
      footerEvents,
      footerClasses,
      footerStyle,
      footerAttrs
    } = column;
    const delegateEvents = this.delegate(footerEvents);

    const cellAttrs = _objectSpread2(_objectSpread2({}, _.isFunction(footerAttrs) ? footerAttrs(column, index) : footerAttrs), delegateEvents);

    let text = '';

    if (_.isString(footer)) {
      text = footer;
    } else if (_.isFunction(footer)) {
      text = footer(columnData, column, index);
    }

    let cellStyle = {};
    const cellClasses = _.isFunction(footerClasses) ? footerClasses(column, index) : footerClasses;

    if (footerStyle) {
      cellStyle = _.isFunction(footerStyle) ? footerStyle(column, index) : footerStyle;
      cellStyle = cellStyle ? _objectSpread2({}, cellStyle) : cellStyle;
    }

    if (footerTitle) {
      cellAttrs.title = _.isFunction(footerTitle) ? footerTitle(column, index) : text;
    }

    if (footerAlign) {
      cellStyle.textAlign = _.isFunction(footerAlign) ? footerAlign(column, index) : footerAlign;
    }

    if (cellClasses) cellAttrs.className = cs__default['default'](cellAttrs.className, cellClasses);
    if (!_.isEmptyObject(cellStyle)) cellAttrs.style = cellStyle;
    const children = footerFormatter ? footerFormatter(column, index, {
      text
    }) : text;
    return /*#__PURE__*/React__default['default'].createElement('th', cellAttrs, children);
  }

}

FooterCell.propTypes = {
  columnData: PropTypes__default['default'].array,
  index: PropTypes__default['default'].number,
  column: PropTypes__default['default'].object
};

/* eslint react/require-default-props: 0 */

const Footer = props => {
  const {
    data,
    className,
    columns,
    selectRow,
    expandRow
  } = props;

  function renderContent() {
    return columns.map((column, i) => {
      if (column.footer === undefined || column.footer === null) {
        return false;
      }

      const columnData = _.pluck(data, column.dataField);

      return /*#__PURE__*/React__default['default'].createElement(FooterCell, {
        index: i,
        key: column.dataField,
        column: column,
        columnData: columnData
      });
    });
  }

  return /*#__PURE__*/React__default['default'].createElement("tfoot", null, /*#__PURE__*/React__default['default'].createElement(RowTemplate, {
    renderContent: renderContent,
    selectRow: selectRow,
    expandRow: expandRow,
    className: className,
    cellEl: "th"
  }));
};

Footer.propTypes = {
  data: PropTypes__default['default'].array,
  className: PropTypes__default['default'].string,
  columns: PropTypes__default['default'].array,
  selectRow: PropTypes__default['default'].object,
  expandRow: PropTypes__default['default'].object
};

var ColumnResolver = (ExtendBase => class ColumnResolver extends ExtendBase {
  visibleColumnSize(includeSelectColumn = true) {
    let columnLen;

    if (this.props.columnToggle && this.props.columnToggle.toggles) {
      const columns = this.props.columnToggle.toggles;
      columnLen = Object.keys(columns).filter(name => columns[name]).length;
    } else {
      columnLen = this.props.columns.filter(c => !c.hidden).length;
    }

    if (!includeSelectColumn) return columnLen;

    if (this.props.selectRow && !this.props.selectRow.hideSelectColumn) {
      columnLen += 1;
    }

    if (this.props.expandRow && this.props.expandRow.showExpandColumn) {
      columnLen += 1;
    }

    return columnLen;
  }

});

var PropsBaseResolver = (ExtendBase => class TableResolver extends ColumnResolver(ExtendBase) {
  validateProps() {
    const {
      keyField
    } = this.props;

    if (!keyField) {
      throw new Error('Please specify a field as key via keyField');
    }

    if (this.visibleColumnSize(false) <= 0) {
      throw new Error('No visible columns detected');
    }
  }

  isEmpty() {
    return this.props.data.length === 0;
  }

  visibleRows() {
    const {
      data,
      hiddenRows,
      keyField
    } = this.props;
    if (!hiddenRows || hiddenRows.length === 0) return data;
    return data.filter(row => {
      const key = _.get(row, keyField);

      return !_.contains(hiddenRows, key);
    });
  }

});

/* eslint-disable import/no-named-as-default-member */

class BootstrapTable extends PropsBaseResolver(React.Component) {
  constructor(props) {
    super(props);

    this.getData = () => {
      return this.visibleRows();
    };

    this.validateProps();
  } // UNSAFE_componentWillReceiveProps(nextProps) {


  componentDidUpdate(prevProps) {
    if (this.props.onDataSizeChange && !this.props.pagination) {
      if (this.props.data.length !== prevProps.data.length) {
        this.props.onDataSizeChange({
          dataSize: this.props.data.length
        });
      }
    }
  } // Exposed APIs


  render() {
    const {
      loading,
      overlay
    } = this.props;

    if (overlay) {
      const LoadingOverlay = overlay(loading);
      return /*#__PURE__*/React__default['default'].createElement(LoadingOverlay, null, this.renderTable());
    }

    return this.renderTable();
  }

  renderTable() {
    const {
      columns,
      keyField,
      tabIndexCell,
      id,
      classes,
      bootstrap4,
      striped,
      hover,
      bordered,
      condensed,
      noDataIndication,
      caption,
      rowStyle,
      rowClasses,
      wrapperClasses,
      rowEvents,
      selectRow,
      expandRow,
      cellEdit,
      filterPosition
    } = this.props;
    const tableWrapperClass = cs__default['default']('react-bootstrap-table', wrapperClasses);
    const tableClass = cs__default['default']('table', {
      'table-striped': striped,
      'table-hover': hover,
      'table-bordered': bordered,
      [bootstrap4 ? 'table-sm' : 'table-condensed']: condensed
    }, classes);
    const hasFilters = columns.some(col => col.filter || col.filterRenderer);
    const hasFooter = _.filter(columns, col => _.has(col, 'footer')).length > 0;
    const tableCaption = caption && /*#__PURE__*/React__default['default'].createElement(Caption, {
      bootstrap4: bootstrap4
    }, caption);
    return /*#__PURE__*/React__default['default'].createElement("div", {
      className: tableWrapperClass
    }, /*#__PURE__*/React__default['default'].createElement("table", {
      id: id,
      className: tableClass
    }, tableCaption, /*#__PURE__*/React__default['default'].createElement(Header, {
      columns: columns,
      className: this.props.headerClasses,
      wrapperClasses: this.props.headerWrapperClasses,
      sortField: this.props.sortField,
      sortOrder: this.props.sortOrder,
      onSort: this.props.onSort,
      globalSortCaret: this.props.sort && this.props.sort.sortCaret,
      onFilter: this.props.onFilter,
      currFilters: this.props.currFilters,
      onExternalFilter: this.props.onExternalFilter,
      selectRow: selectRow,
      expandRow: expandRow,
      filterPosition: filterPosition
    }), hasFilters && filterPosition !== Const.FILTERS_POSITION_INLINE && /*#__PURE__*/React__default['default'].createElement(Filters, {
      columns: columns,
      className: this.props.filtersClasses,
      onSort: this.props.onSort,
      onFilter: this.props.onFilter,
      currFilters: this.props.currFilters,
      filterPosition: this.props.filterPosition,
      onExternalFilter: this.props.onExternalFilter,
      selectRow: selectRow,
      expandRow: expandRow
    }), /*#__PURE__*/React__default['default'].createElement(Body, {
      className: this.props.bodyClasses,
      data: this.getData(),
      keyField: keyField,
      tabIndexCell: tabIndexCell,
      columns: columns,
      isEmpty: this.isEmpty(),
      visibleColumnSize: this.visibleColumnSize(),
      noDataIndication: noDataIndication,
      cellEdit: cellEdit,
      selectRow: selectRow,
      expandRow: expandRow,
      rowStyle: rowStyle,
      rowClasses: rowClasses,
      rowEvents: rowEvents
    }), hasFooter && /*#__PURE__*/React__default['default'].createElement(Footer, {
      data: this.getData(),
      columns: columns,
      selectRow: selectRow,
      expandRow: expandRow,
      className: this.props.footerClasses
    })));
  }

}

BootstrapTable.propTypes = {
  keyField: PropTypes__default['default'].string.isRequired,
  data: PropTypes__default['default'].array.isRequired,
  columns: PropTypes__default['default'].array.isRequired,
  bootstrap4: PropTypes__default['default'].bool,
  remote: PropTypes__default['default'].oneOfType([PropTypes__default['default'].bool, PropTypes__default['default'].shape({
    pagination: PropTypes__default['default'].bool
  })]),
  noDataIndication: PropTypes__default['default'].oneOfType([PropTypes__default['default'].node, PropTypes__default['default'].func]),
  striped: PropTypes__default['default'].bool,
  bordered: PropTypes__default['default'].bool,
  hover: PropTypes__default['default'].bool,
  tabIndexCell: PropTypes__default['default'].bool,
  id: PropTypes__default['default'].string,
  classes: PropTypes__default['default'].string,
  headerClasses: PropTypes__default['default'].string,
  bodyClasses: PropTypes__default['default'].string,
  wrapperClasses: PropTypes__default['default'].string,
  headerWrapperClasses: PropTypes__default['default'].string,
  condensed: PropTypes__default['default'].bool,
  caption: PropTypes__default['default'].oneOfType([PropTypes__default['default'].node, PropTypes__default['default'].string]),
  pagination: PropTypes__default['default'].object,
  filter: PropTypes__default['default'].object,
  cellEdit: PropTypes__default['default'].object,
  selectRow: PropTypes__default['default'].shape({
    mode: PropTypes__default['default'].oneOf([Const.ROW_SELECT_SINGLE, Const.ROW_SELECT_MULTIPLE, Const.ROW_SELECT_DISABLED]).isRequired,
    clickToSelect: PropTypes__default['default'].bool,
    clickToExpand: PropTypes__default['default'].bool,
    clickToEdit: PropTypes__default['default'].bool,
    hideSelectAll: PropTypes__default['default'].bool,
    onSelect: PropTypes__default['default'].func,
    onSelectAll: PropTypes__default['default'].func,
    style: PropTypes__default['default'].oneOfType([PropTypes__default['default'].object, PropTypes__default['default'].func]),
    classes: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].func]),
    nonSelectable: PropTypes__default['default'].array,
    nonSelectableStyle: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].func]),
    nonSelectableClasses: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].func]),
    bgColor: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].func]),
    hideSelectColumn: PropTypes__default['default'].bool,
    selectionRenderer: PropTypes__default['default'].func,
    selectionHeaderRenderer: PropTypes__default['default'].func,
    headerColumnStyle: PropTypes__default['default'].oneOfType([PropTypes__default['default'].object, PropTypes__default['default'].func]),
    selectColumnStyle: PropTypes__default['default'].oneOfType([PropTypes__default['default'].object, PropTypes__default['default'].func]),
    selectColumnPosition: PropTypes__default['default'].oneOf([Const.INDICATOR_POSITION_LEFT, Const.INDICATOR_POSITION_RIGHT])
  }),
  expandRow: PropTypes__default['default'].shape({
    renderer: PropTypes__default['default'].func,
    expanded: PropTypes__default['default'].array,
    onExpand: PropTypes__default['default'].func,
    onExpandAll: PropTypes__default['default'].func,
    nonExpandable: PropTypes__default['default'].array,
    showExpandColumn: PropTypes__default['default'].bool,
    onlyOneExpanding: PropTypes__default['default'].bool,
    expandByColumnOnly: PropTypes__default['default'].bool,
    expandColumnRenderer: PropTypes__default['default'].func,
    expandHeaderColumnRenderer: PropTypes__default['default'].func,
    expandColumnPosition: PropTypes__default['default'].oneOf([Const.INDICATOR_POSITION_LEFT, Const.INDICATOR_POSITION_RIGHT]),
    className: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].func]),
    parentClassName: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].func])
  }),
  rowStyle: PropTypes__default['default'].oneOfType([PropTypes__default['default'].object, PropTypes__default['default'].func]),
  rowEvents: PropTypes__default['default'].object,
  rowClasses: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].func]),
  filtersClasses: PropTypes__default['default'].string,
  filterPosition: PropTypes__default['default'].oneOf([Const.FILTERS_POSITION_TOP, Const.FILTERS_POSITION_INLINE, Const.FILTERS_POSITION_BOTTOM]),
  footerClasses: PropTypes__default['default'].string,
  defaultSorted: PropTypes__default['default'].arrayOf(PropTypes__default['default'].shape({
    dataField: PropTypes__default['default'].string.isRequired,
    order: PropTypes__default['default'].oneOf([Const.SORT_DESC, Const.SORT_ASC]).isRequired
  })),
  sort: PropTypes__default['default'].shape({
    dataField: PropTypes__default['default'].string,
    order: PropTypes__default['default'].oneOf([Const.SORT_DESC, Const.SORT_ASC]),
    sortFunc: PropTypes__default['default'].func,
    sortCaret: PropTypes__default['default'].func
  }),
  defaultSortDirection: PropTypes__default['default'].oneOf([Const.SORT_DESC, Const.SORT_ASC]),
  overlay: PropTypes__default['default'].func,
  onTableChange: PropTypes__default['default'].func,
  onSort: PropTypes__default['default'].func,
  onFilter: PropTypes__default['default'].func,
  onExternalFilter: PropTypes__default['default'].func,
  onDataSizeChange: PropTypes__default['default'].func,
  // Inject from toolkit
  search: PropTypes__default['default'].shape({
    searchText: PropTypes__default['default'].string,
    searchContext: PropTypes__default['default'].func
  }),
  setDependencyModules: PropTypes__default['default'].func
};
BootstrapTable.defaultProps = {
  bootstrap4: false,
  remote: false,
  striped: false,
  bordered: true,
  hover: false,
  condensed: false,
  noDataIndication: null,
  selectRow: {
    mode: Const.ROW_SELECT_DISABLED,
    selected: [],
    hideSelectColumn: true
  },
  expandRow: {
    renderer: undefined,
    expanded: [],
    nonExpandable: []
  },
  cellEdit: {
    mode: null,
    nonEditableRows: []
  },
  filterPosition: Const.FILTERS_POSITION_INLINE
};

function _classPrivateFieldBase(receiver, privateKey) {
  if (!Object.prototype.hasOwnProperty.call(receiver, privateKey)) {
    throw new TypeError("attempted to use private field on non-instance");
  }

  return receiver;
}

var id = 0;
function _classPrivateFieldKey(name) {
  return "__private_" + id++ + "_" + name;
}

var createDataContext = (() => {
  const DataContext = /*#__PURE__*/React__default['default'].createContext();

  var _data = _classPrivateFieldKey("data");

  class DataProvider extends React.Component {
    constructor(...args) {
      super(...args);
      Object.defineProperty(this, _data, {
        writable: true,
        value: this.props.data
      });

      this.getData = (filterProps, searchProps, sortProps, paginationProps) => {
        if (paginationProps) return paginationProps.data;
        if (sortProps) return sortProps.data;
        if (searchProps) return searchProps.data;
        if (filterProps) return filterProps.data;
        return this.data;
      };
    }

    // eslint-disable-next-line react/state-in-constructor, react/destructuring-assignment
    // state = { data: this.props.data };
    // componentDidUpdate(prevProps) {
    //   if (!_.isEqual(this.props.data, prevProps.data)) {
    //     // eslint-disable-next-line react/no-did-update-set-state
    //     this.setState({ data: this.props.data });
    //   }
    // }
    get data() {
      return _classPrivateFieldBase(this, _data)[_data];
    }

    set data(value) {
      _classPrivateFieldBase(this, _data)[_data] = value;
    }

    render() {
      const {
        children
      } = this.props;
      return /*#__PURE__*/React__default['default'].createElement(DataContext.Provider, {
        value: {
          data: this.data,
          getData: this.getData
        }
      }, children);
    }

  }

  DataProvider.propTypes = {
    data: PropTypes__default['default'].array.isRequired,
    children: PropTypes__default['default'].node.isRequired
  };
  return {
    Provider: DataProvider,
    Consumer: DataContext.Consumer
  };
});

/* eslint react/prop-types: 0 */
var createColumnMgtContext = (() => {
  const ColumnManagementContext = /*#__PURE__*/React__default['default'].createContext();

  class ColumnManagementProvider extends React__default['default'].Component {
    render() {
      let toggleColumn;
      const {
        columns,
        toggles,
        children
      } = this.props;

      if (toggles) {
        toggleColumn = columns.filter(column => toggles[column.dataField]);
      } else {
        toggleColumn = columns.filter(column => !column.hidden);
      }

      return /*#__PURE__*/React__default['default'].createElement(ColumnManagementContext.Provider, {
        value: {
          columns: toggleColumn
        }
      }, children);
    }

  }

  ColumnManagementProvider.propTypes = {
    columns: PropTypes__default['default'].array.isRequired,
    toggles: PropTypes__default['default'].object
  };
  ColumnManagementProvider.defaultProps = {
    toggles: null
  };
  return {
    Provider: ColumnManagementProvider,
    Consumer: ColumnManagementContext.Consumer
  };
});

var createSortContext = ((dataOperator, isRemoteSort, handleSortChange) => {
  const SortContext = /*#__PURE__*/React__default['default'].createContext();

  class SortProvider extends React__default['default'].Component {
    constructor(props) {
      super(props);

      this.handleSort = column => {
        const {
          defaultSortDirection
        } = this.props;
        const sortOrder = dataOperator.nextOrder(column, this.state, defaultSortDirection);

        if (column.onSort) {
          column.onSort(column.dataField, sortOrder);
        }

        if (isRemoteSort()) {
          handleSortChange(column.dataField, sortOrder);
        }

        this.setState(() => ({
          sortOrder,
          sortColumn: column
        }));
      };

      let _sortOrder;

      let sortColumn;
      const {
        defaultSorted,
        defaultSortDirection: _defaultSortDirection,
        sort
      } = props;

      if (defaultSorted && defaultSorted.length > 0) {
        _sortOrder = defaultSorted[0].order || _defaultSortDirection;
        sortColumn = this.initSort(defaultSorted[0].dataField, _sortOrder);
      } else if (sort && sort.dataField && sort.order) {
        _sortOrder = sort.order;
        sortColumn = this.initSort(sort.dataField, _sortOrder);
      }

      this.state = {
        sortOrder: _sortOrder,
        sortColumn
      };
    }

    componentDidMount() {
      const {
        sortOrder,
        sortColumn
      } = this.state;

      if (isRemoteSort() && sortOrder && sortColumn) {
        handleSortChange(sortColumn.dataField, sortOrder);
      }
    } // UNSAFE_componentWillReceiveProps(nextProps) {


    static getDerivedStateFromProps(nextProps) {
      var _nextProps$sort, _nextProps$sort2;

      if (nextProps !== null && nextProps !== void 0 && (_nextProps$sort = nextProps.sort) !== null && _nextProps$sort !== void 0 && _nextProps$sort.dataField && nextProps !== null && nextProps !== void 0 && (_nextProps$sort2 = nextProps.sort) !== null && _nextProps$sort2 !== void 0 && _nextProps$sort2.order) {
        const {
          sort,
          columns
        } = nextProps;
        return {
          sortOrder: sort.order,
          sortColumn: columns.find(col => col.dataField === sort.dataField)
        };
      }

      return null;
    }

    initSort(sortField, sortOrder) {
      let sortColumn;
      const {
        columns
      } = this.props;
      const [sortColumns] = columns.filter(col => col.dataField === sortField);

      if (sortColumns.length > 0) {
        sortColumn = sortColumns;

        if (sortColumn.onSort) {
          sortColumn.onSort(sortField, sortOrder);
        }
      }

      return sortColumn;
    }

    render() {
      let {
        data
      } = this.props;
      const {
        children
      } = this.props;
      const {
        sort
      } = this.props;
      const {
        sortOrder,
        sortColumn
      } = this.state;

      if (!isRemoteSort() && sortColumn) {
        const sortFunc = sortColumn.sortFunc ? sortColumn.sortFunc : sort && sort.sortFunc;
        data = dataOperator.sort(data, sortOrder, _objectSpread2(_objectSpread2({}, sortColumn), {}, {
          sortFunc
        }));
      }

      return /*#__PURE__*/React__default['default'].createElement(SortContext.Provider, {
        value: {
          data,
          sortOrder,
          onSort: this.handleSort,
          sortField: sortColumn ? sortColumn.dataField : null
        }
      }, children);
    }

  }

  SortProvider.propTypes = {
    data: PropTypes__default['default'].array.isRequired,
    columns: PropTypes__default['default'].array.isRequired,
    children: PropTypes__default['default'].node.isRequired,
    defaultSorted: PropTypes__default['default'].arrayOf(PropTypes__default['default'].shape({
      dataField: PropTypes__default['default'].string.isRequired,
      order: PropTypes__default['default'].oneOf([Const.SORT_DESC, Const.SORT_ASC]).isRequired
    })),
    sort: PropTypes__default['default'].shape({
      dataField: PropTypes__default['default'].string,
      order: PropTypes__default['default'].oneOf([Const.SORT_DESC, Const.SORT_ASC]),
      sortFunc: PropTypes__default['default'].func
    }),
    defaultSortDirection: PropTypes__default['default'].oneOf([Const.SORT_DESC, Const.SORT_ASC])
  };
  return {
    Provider: SortProvider,
    Consumer: SortContext.Consumer
  };
});

var remoteResolver = (ExtendBase => {
  var _temp;

  return _temp = class RemoteResolver extends ExtendBase {
    constructor(props) {
      super(props);

      this.getNewestState = (state = {}) => {
        let sortOrder;
        let sortField;
        let page;
        let sizePerPage;
        let searchText;
        let filters = {};

        if (this.sortContext) {
          sortOrder = this.sortContext.state.sortOrder;
          sortField = this.sortContext.state.sortColumn ? this.sortContext.state.sortColumn.dataField : null;
        }

        if (this.filterContext) {
          filters = this.filterContext.currFilters;
        }

        if (this.paginationContext) {
          page = this.paginationContext.currPage;
          sizePerPage = this.paginationContext.currSizePerPage;
        }

        if (this.searchContext) {
          searchText = this.props.search.searchText;
        }

        return _objectSpread2(_objectSpread2({
          sortOrder,
          sortField,
          filters,
          page,
          sizePerPage,
          searchText
        }, state), {}, {
          data: this.props.data
        });
      };

      this.isRemoteSearch = () => {
        const {
          remote
        } = this.props;
        return remote === true || _.isObject(remote) && remote.search || this.isRemotePagination();
      };

      this.isRemotePagination = (e = {}) => {
        const {
          remote
        } = this.props;
        e.result = remote === true || _.isObject(remote) && remote.pagination;
        return e.result;
      };

      this.isRemoteFiltering = () => {
        const {
          remote
        } = this.props;
        return remote === true || _.isObject(remote) && remote.filter || this.isRemotePagination();
      };

      this.isRemoteSort = () => {
        const {
          remote
        } = this.props;
        return remote === true || _.isObject(remote) && remote.sort || this.isRemotePagination();
      };

      this.isRemoteCellEdit = () => {
        const {
          remote
        } = this.props;
        return remote === true || _.isObject(remote) && remote.cellEdit;
      };

      this.handleRemotePageChange = (page, sizePerPage) => {
        this.props.onTableChange('pagination', this.getNewestState({
          page,
          sizePerPage
        }));
      };

      this.handleRemoteFilterChange = filters => {
        const newState = {
          filters
        };

        if (this.isRemotePagination()) {
          const options = this.props.pagination.options || {};
          newState.page = _.isDefined(options.pageStartIndex) ? options.pageStartIndex : 1;
        }

        this.props.onTableChange('filter', this.getNewestState(newState));
      };

      this.handleRemoteSortChange = (sortField, sortOrder) => {
        this.props.onTableChange('sort', this.getNewestState({
          sortField,
          sortOrder
        }));
      };

      this.handleRemoteCellChange = (rowId, dataField, newValue) => {
        const cellEdit = {
          rowId,
          dataField,
          newValue
        };
        this.props.onTableChange('cellEdit', this.getNewestState({
          cellEdit
        }));
      };

      this.handleRemoteSearchChange = searchText => {
        this.props.onTableChange('search', this.getNewestState({
          searchText
        }));
      };

      this.remoteEmitter = new EventEmitter__default['default']();
      this.remoteEmitter.on('paginationChange', this.handleRemotePageChange);
      this.remoteEmitter.on('isRemotePagination', this.isRemotePagination);
    }

  }, _temp;
});

const withContext = Base => class BootstrapTableContainer extends remoteResolver(React.Component) {
  constructor(props) {
    super(props);
    this.DataContext = createDataContext();

    if (props.registerExposedAPI) {
      const exposedAPIEmitter = new EventEmitter__default['default']();
      exposedAPIEmitter.on('get.table.data', payload => payload.result = this.table.getData());
      exposedAPIEmitter.on('get.selected.rows', payload => payload.result = this.selectionContext.getSelected());
      exposedAPIEmitter.on('get.filtered.rows', payload => {
        if (this.searchContext) {
          payload.result = this.searchContext.getSearched();
        } else if (this.filterContext) {
          payload.result = this.filterContext.getFiltered();
        } else {
          payload.result = this.table.getData();
        }
      });
      props.registerExposedAPI(exposedAPIEmitter);
    }

    if (props.columns.filter(col => col.sort).length > 0) {
      this.SortContext = createSortContext(dataOperator, this.isRemoteSort, this.handleRemoteSortChange);
    }

    if (props.columnToggle || props.columns.filter(col => col.hidden).length > 0) {
      this.ColumnManagementContext = createColumnMgtContext();
    }

    if (props.selectRow) {
      this.SelectionContext = SelectionContext$1;
    }

    if (props.expandRow) {
      this.RowExpandContext = RowExpandContext$1;
    }

    if (props.cellEdit && props.cellEdit.createContext) {
      this.CellEditContext = props.cellEdit.createContext(_, dataOperator, this.isRemoteCellEdit, this.handleRemoteCellChange);
    }

    if (props.filter) {
      this.FilterContext = props.filter.createContext(_, this.isRemoteFiltering, this.handleRemoteFilterChange);
    }

    if (props.pagination) {
      this.PaginationContext = props.pagination.createContext();
    }

    if (props.search && props.search.searchContext) {
      this.SearchContext = props.search.searchContext(_, this.isRemoteSearch, this.handleRemoteSearchChange);
    }

    if (props.setDependencyModules) {
      props.setDependencyModules(_);
    }

    if (props.setPaginationRemoteEmitter) {
      props.setPaginationRemoteEmitter(this.remoteEmitter);
    }
  } // UNSAFE_componentWillReceiveProps(nextProps) {


  componentDidUpdate(prevProps) {
    if (this.props.columns.filter(col => col.sort).length <= 0) {
      this.SortContext = null;
    } else if (!this.SortContext) {
      this.SortContext = createSortContext(dataOperator, this.isRemoteSort, this.handleRemoteSortChange);
    }

    if (!this.props.pagination && prevProps.pagination) {
      this.PaginationContext = null;
    }

    if (this.props.pagination && !prevProps.pagination) {
      this.PaginationContext = this.props.pagination.createContext(this.isRemotePagination, this.handleRemotePageChange);
    }

    if (!this.props.cellEdit && prevProps.cellEdit) {
      this.CellEditContext = null;
    }

    if (this.props.cellEdit && !prevProps.cellEdit) {
      this.CellEditContext = this.props.cellEdit.createContext(_, dataOperator, this.isRemoteCellEdit, this.handleRemoteCellChange);
    }
  }

  renderBase() {
    return (rootProps, filterProps, searchProps, sortProps, paginationProps, columnToggleProps) => /*#__PURE__*/React__default['default'].createElement(Base, _extends({
      ref: n => this.table = n
    }, this.props, sortProps, filterProps, searchProps, paginationProps, columnToggleProps, {
      data: rootProps.getData(filterProps, searchProps, sortProps, paginationProps)
    }));
  }

  renderWithColumnManagementCtx(base, baseProps) {
    const {
      columnToggle
    } = this.props;
    return (rootProps, filterProps, searchProps, sortProps, paginationProps) => /*#__PURE__*/React__default['default'].createElement(this.ColumnManagementContext.Provider, _extends({}, baseProps, {
      toggles: columnToggle ? columnToggle.toggles : null
    }), /*#__PURE__*/React__default['default'].createElement(this.ColumnManagementContext.Consumer, null, columnToggleProps => base(rootProps, filterProps, searchProps, sortProps, paginationProps, columnToggleProps)));
  }

  renderWithSelectionCtx(base, baseProps) {
    const {
      selectRow
    } = this.props;
    return (rootProps, filterProps, searchProps, sortProps, paginationProps) => /*#__PURE__*/React__default['default'].createElement(this.SelectionContext.Provider, _extends({}, baseProps, {
      ref: n => this.selectionContext = n,
      selectRow: selectRow,
      data: rootProps.getData(filterProps, searchProps, sortProps, paginationProps)
    }), base(rootProps, filterProps, searchProps, sortProps, paginationProps));
  }

  renderWithRowExpandCtx(base, baseProps) {
    const {
      expandRow
    } = this.props;
    return (rootProps, filterProps, searchProps, sortProps, paginationProps) => /*#__PURE__*/React__default['default'].createElement(this.RowExpandContext.Provider, _extends({}, baseProps, {
      ref: n => this.rowExpandContext = n,
      expandRow: expandRow,
      data: rootProps.getData(filterProps, searchProps, sortProps, paginationProps)
    }), base(rootProps, filterProps, searchProps, sortProps, paginationProps));
  }

  renderWithPaginationCtx(base) {
    const {
      pagination,
      bootstrap4,
      onDataSizeChange,
      id
    } = this.props;
    return (rootProps, filterProps, searchProps, sortProps) => /*#__PURE__*/React__default['default'].createElement(this.PaginationContext.Provider, {
      ref: n => this.paginationContext = n,
      pagination: pagination,
      data: rootProps.getData(filterProps, searchProps, sortProps),
      bootstrap4: bootstrap4,
      isRemotePagination: this.isRemotePagination,
      remoteEmitter: this.remoteEmitter,
      onDataSizeChange: onDataSizeChange,
      tableId: id
    }, /*#__PURE__*/React__default['default'].createElement(this.PaginationContext.Consumer, null, paginationProps => base(rootProps, filterProps, searchProps, sortProps, paginationProps)));
  }

  renderWithSortCtx(base, baseProps) {
    const {
      defaultSorted,
      defaultSortDirection,
      sort
    } = this.props;
    return (rootProps, filterProps, searchProps) => /*#__PURE__*/React__default['default'].createElement(this.SortContext.Provider, _extends({}, baseProps, {
      ref: n => this.sortContext = n,
      defaultSorted: defaultSorted,
      defaultSortDirection: defaultSortDirection,
      sort: sort,
      data: rootProps.getData(filterProps, searchProps)
    }), /*#__PURE__*/React__default['default'].createElement(this.SortContext.Consumer, null, sortProps => base(rootProps, filterProps, searchProps, sortProps)));
  }

  renderWithSearchCtx(base, baseProps) {
    const {
      search,
      dataChangeListener
    } = this.props;
    return (rootProps, filterProps) => /*#__PURE__*/React__default['default'].createElement(this.SearchContext.Provider, _extends({}, baseProps, {
      ref: n => this.searchContext = n,
      data: rootProps.getData(filterProps),
      searchText: search.searchText,
      dataChangeListener: dataChangeListener
    }), /*#__PURE__*/React__default['default'].createElement(this.SearchContext.Consumer, null, searchProps => base(rootProps, filterProps, searchProps)));
  }

  renderWithFilterCtx(base, baseProps) {
    const {
      filter,
      dataChangeListener
    } = this.props;
    return rootProps => /*#__PURE__*/React__default['default'].createElement(this.FilterContext.Provider, _extends({}, baseProps, {
      ref: n => this.filterContext = n,
      data: rootProps.getData(),
      filter: filter.options || {},
      dataChangeListener: dataChangeListener
    }), /*#__PURE__*/React__default['default'].createElement(this.FilterContext.Consumer, null, filterProps => base(rootProps, filterProps)));
  }

  renderWithCellEditCtx(base, baseProps) {
    const {
      selectRow,
      cellEdit
    } = this.props;
    return rootProps => /*#__PURE__*/React__default['default'].createElement(this.CellEditContext.Provider, _extends({}, baseProps, {
      ref: n => this.cellEditContext = n,
      selectRow: selectRow,
      cellEdit: cellEdit,
      data: rootProps.getData()
    }), base(rootProps));
  }

  render() {
    const {
      keyField,
      columns,
      bootstrap4
    } = this.props;
    const baseProps = {
      keyField,
      columns
    };
    let base = this.renderBase();

    if (this.ColumnManagementContext) {
      base = this.renderWithColumnManagementCtx(base, baseProps);
    }

    if (this.SelectionContext) {
      base = this.renderWithSelectionCtx(base, baseProps);
    }

    if (this.RowExpandContext) {
      base = this.renderWithRowExpandCtx(base, baseProps);
    }

    if (this.PaginationContext) {
      base = this.renderWithPaginationCtx(base, baseProps);
    }

    if (this.SortContext) {
      base = this.renderWithSortCtx(base, baseProps);
    }

    if (this.SearchContext) {
      base = this.renderWithSearchCtx(base, baseProps);
    }

    if (this.FilterContext) {
      base = this.renderWithFilterCtx(base, baseProps);
    }

    if (this.CellEditContext) {
      base = this.renderWithCellEditCtx(base, baseProps);
    }

    return /*#__PURE__*/React__default['default'].createElement(BootstrapContext.Provider, {
      value: {
        bootstrap4
      }
    }, /*#__PURE__*/React__default['default'].createElement(this.DataContext.Provider, _extends({}, baseProps, {
      data: this.props.data
    }), /*#__PURE__*/React__default['default'].createElement(this.DataContext.Consumer, null, base)));
  }

};

var index = withContext(BootstrapTable);

exports.default = index;
